
/*******************************************************************************
*	DISCLAIMER
*
*	THIS SOFTWARE IS PROVIDED "AS IS" AND BFG ENGINEERING MAKES NO WARRANTIES
*	REGARDING THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING
*	BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
*	PURPOSE AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
*
*	WE (BFG ENGINEERING) DO NOT WARRANT THAT THIS SOFTWARE IS FREE FROM CLAIMS
*	BY A THIRD PARTY OF COPYRIGHT, PATENT, TRADEMARK, TRADE SECRET OR ANY OTHER
*	INTELLECTUAL PROPERTY INFRINGIMENT.
*
*	TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, BFG ENGINEERING
*	SHALL NOT BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR
*	CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS SOFTWARE, EVEN IF BFG
*	ENGINEERING HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
*	PARTICULARLY, UNDER NO CIRCUMSTANCES ARE WE LIABLE FOR ANY OF THE FOLLOWING:
*	- THIRD PARTY CLAIMS AGAINST YOU FOR LOSSES OR DAMAGES;
*	- LOSS OF, OR DAMAGE TO YOUR RECORDS OR DATA; OR
*	- ECONOMIC CONSEQUENTIAL DAMAGES (INCLUDING LOST PROFITS OR SAVINGS) OR
*	  INCIDENTAL DAMAGES, EVEN IF WE ARE INFORMED OF THEIR POSSIBILITY.
*
*	BFG ENGINEERING DOES NOT WARRANT UNINTERRUPTED OR ERROR_FREE OPERATION OF
*	THIS SOFTWARE.
*	BFG ENGINEERING HAS NO OBLIGATION TO PROVIDE SERVICE, DEFECT CORRECTION OR
*	ANY MANTEINANCE OF THIS SOFTWARE; OR TO SUPPLY ANY UPDATES OR ENHANCEMENTS
*	EVEN IF SUCH ARE, OR LATER BECOME, AVAILABLE.
*
*	IF YOU DOWNLOAD OR USE THIS SOFTWARE YOU AGREE TO THESE TERMS, AND DO IT AT
*	YOUR OWN RISK.
*******************************************************************************/

/*******************************************************************************
*	File:		motorcontrol.c
*	Project:	MCRP07 RX62T
*	Revision:	7.0
*	Date:		25-10-2013 (DD-MM-YYYY)
*	Author:		Andrea Ghirardello
*	Contents:	Motor control algorithm
*******************************************************************************/

#include <mathf.h>
#include "globdef.h"
#include "ges_eqp.h"
#include "userif.h"
#include "motorcontrol.h"
#include ".\MCRP07_Library\mcrplibf.h"
#ifdef CPI_AUT
#include ".\MCRP07_Library\PI_tun.h"
#endif	// CPI_AUT
#ifdef	MOT_IDENT
#include ".\MCRP07_Library\mot_ident.h"
#endif	// MOT_IDENT
#ifdef INLCOMP
#ifdef INL_DET
#include ".\MCRP07_Library\inv_nl.h"
#endif	// INL_DET
#endif	// INLCOMP

// flags
#define	COM_REQ			(flgx.bit.b0!=0)
#define	FREE01			(flgx.bit.b1!=0)
#define	FREE02			(flgx.bit.b2!=0)
#define	RAMPLIM			(flgx.bit.b3!=0)
#define	DECEL			(flgx.bit.b4!=0)
#define	FREE05			(flgx.bit.b5!=0)
#define	STOPPED			(flgx.bit.b6!=0)
#define	STARTED			(flgx.bit.b7!=0)
#define	NOT_COM_REQ		(flgx.bit.b0==0)
#define	NOT_FREE01		(flgx.bit.b1==0)
#define	NOT_FREE02		(flgx.bit.b2==0)
#define	NOT_RAMPLIM		(flgx.bit.b3==0)
#define	NOT_DECEL		(flgx.bit.b4==0)
#define	NOT_FREE05		(flgx.bit.b5==0)
#define	NOT_STOPPED		(flgx.bit.b6==0)
#define	NOT_STARTED		(flgx.bit.b7==0)
#define	SET_COM_REQ		flgx.bit.b0=1;
#define	SET_FREE01		flgx.bit.b1=1;
#define	SET_FREE02		flgx.bit.b2=1;
#define	SET_RAMPLIM		flgx.bit.b3=1;
#define	SET_DECEL		flgx.bit.b4=1;
#define	SET_FREE05		flgx.bit.b5=1;
#define	SET_STOPPED		flgx.bit.b6=1;
#define	SET_STARTED		flgx.bit.b7=1;
#define	RES_COM_REQ		flgx.bit.b0=0;
#define	RES_FREE01		flgx.bit.b1=0;
#define	RES_FREE02		flgx.bit.b2=0;
#define	RES_RAMPLIM		flgx.bit.b3=0;
#define	RES_DECEL		flgx.bit.b4=0;
#define	RES_FREE05		flgx.bit.b5=0;
#define	RES_STOPPED		flgx.bit.b6=0;
#define	RES_STARTED		flgx.bit.b7=0;

// internal filters (main interrupt) time constants	[sec]
#define	VB_KT_FAST		0.002f		// bus voltage filter fast
#define	VB_KT_SLOW		0.2f		// bus voltage filter slow
#define	VM_KT_FAST		0.002f		// voltage amplitude filter fast
#define	VM_KT_SLOW		0.05f		// voltage amplitude filter slow
#define	VR_KT			0.002f		// voltage reference filter
#define	OFF_KT			0.2f		// A/D offsets filters
#define	OMR_KT			0.05f		// speed reference filter
#define VISU_KT			0.3f		// visualization filters

// filters time constant (1/lpkt) (referred to sampling frequency)
#define	VB_UKT_FAST		((float)(1.0/(1.0+(VB_KT_FAST/SAM_PER_S))))
#define	VB_UKT_SLOW		((float)(1.0/(1.0+(VB_KT_SLOW/SAM_PER_S))))
#define	VM_UKT_FAST		((float)(1.0/(1.0+(VM_KT_FAST/SAM_PER_S))))
#define	VM_UKT_SLOW		((float)(1.0/(1.0+(VM_KT_SLOW/SAM_PER_S))))
#define	VR_UKT			((float)(1.0/(1.0+(VR_KT/SAM_PER_S))))
#define	OFF_UKT			((float)(1.0/(1.0+(OFF_KT/SAM_PER_S))))
#define	OMR_UKT			((float)(1.0/(1.0+(OMR_KT/SAM_PER_S))))
#define VISU_UKT		((float)(1.0/(1.0+(VISU_KT/SAM_PER_S))))

// state values for three shunts current reading
#define	CR_NO			0
#define	CR_UV			1
#define	CR_VW			2
#define	CR_WU			3

// speed ramp state machine
#define STOP_0			0
#define STOP_W			1
#define START_P			2
#define START_N			3
#define GO_P			4
#define GO_N			5

// trip alarm speed thresholds
#define T_MARGL			0.15f
#define T_MARGH			1.125f
#define R_MARG			100.0f

// internal variables
byte_t
	flgx,					// flags
	flgy;					// flags

unsigned short
	cnt_int,				// counter for time synchronization
	cr_ts,					// state memory for three shunts current reading
	r_acc_ep,				// acceleration ramp (copy)
	r_dec_ep,				// deceleration ramp (copy)
	i_start_ep,				// startup current (copy)
	i_max_ep,				// max current (copy)
	r_sta_ep,				// stator resistance (copy)
	l_syn_ep,				// synchronous inductance (copy)
	pm_flx_ep,				// permanent magnets flux (copy)
	fb_gain_ep,				// flux amplitude feedback gain (copy)
	kp_cur_ep,				// current PI proportional gain (copy)
	ki_cur_ep,				// current PI integral gain (copy)
	kp_vel_ep,				// speed PI proportional gain (copy)
	ki_vel_ep,				// speed PI integral gain (copy)
	ramp_status=STOP_0;		// speed ramp state machine status memory

signed short
	startup_cnt,			// startup counter
	trip_cnt,				// counter for trip alarm
	duty_u,					// u phase duty cycle
	duty_v,					// v phase duty cycle
	duty_w,					// w phase duty cycle
	iuad,					// iu A/D conversion value
	ivad,					// iv A/D conversion value
	iwad,					// iw A/D conversion value
	vbus_ad;				// bus voltage A/D conversion value

float
	ium_off,				// IU analog channel offset
	ivm_off,				// IV analog channel offset
	iwm_off,				// IW analog channel offset
	vol_ref,				// reference voltage
	k_vref,					// constant related to vol_ref
	kadi,					// current conversion constant, corrected
	kadv,					// voltage conversion constant, corrected
	r_sta,					// stator resistance
	l_syn,					// synchronous inductance
	pm_flx,					// permanent magnets flux
	fb_gain,				// flux amplitude feedback gain
	c_poli,					// number of polar couples
	krpmocp,				// conversion constant between mechanical speed and electrical speed
	ukrpmocp,				// 1/krpmocp
	i_start,				// startup current (peak)
	delta_om,				// minimum speed variation (during startup)
	is_rup,					// startup current increasing rate
	is_rdw,					// startup current decreasing rate
	vbus,					// bus voltage
	vbusf,					// bus voltage (filtered)
	xvb,					// available DC voltage (corrected with IGBT drop)
	xvbf,					// available voltage (minimum value)
	vfmax,					// max phase voltage (star)
	//vdmax,					// maximum d voltage
	//vqmax,					// maximum q voltage
	i_max,					// maximum total current
	id_max,					// maximum d current
	iqmax,					// maximum q current
	vdc,					// applied d voltage
	vqc,					// applied q voltage
	vdm,					// d voltage (applied at previous step)
	vqm,					// q voltage (applied at previous step)
	vdmf,					// applied d voltage (filtered)
	vqmf,					// applied q voltage (filtered)
	vac,					// applied alpha voltage (star)
	vbc,					// applied beta voltage (star)
	vam,					// alpha voltage (applied at previous step)
	vbm,					// beta voltage (applied at previous step)
	vuc,					// applied u voltage (star)
	vvc,					// applied v voltage (star)
	vwc,					// applied w voltage (star)
	vmarg,					// voltage margin for flux weakening
	ium,					// u current (measured)
	ivm,					// v current (measured)
	iwm,					// w current (measured)
	iam,					// alpha current (measured)
	ibm,					// beta current (measured)
	idm,					// d current (measured)
	iqm,					// q current (measured)
	idmf,					// d current (filtered)
	iqmf,					// q current (filtered)
	idr,					// d current reference
	iqr,					// q current reference
	iar,					// alpha current reference
	ibr,					// beta current reference
	idint,					// d current error integral value
	iqint,					// q current error integral value
	iaint,					// a current error integral value
	ibint,					// b current error integral value
	idrvar,					// d current reference
	errint,					// speed error integral memory
	kp_cur,					// current PI proportional gain
	ki_cur,					// current PI integral gain
	kp_vel,					// speed PI proportional gain
	ki_vel,					// speed PI integral gain
	freq,					// electrical frequency
	mec_rpm,				// mechanical speed (rpm)
	rpmrif_x,				// speed reference in rpm (ramp input)
	rpmrif_y,				// speed reference in rpm (ramp output)
	rpmrif_y_abs,			// |speed reference| in rpm (ramp output)
	r_acc,					// acceleration ramp
	r_dec,					// deceleration ramp
#ifdef RAMP_CL0
	r_marg,					// useful in speed thresholds determination
#endif	// RAMP_CL0
	rpm_max,				// maximum speed (rpm)
	rpm_min,				// minimum speed (rpm)
	min_speed,				// minimum speed (rad/s)
	max_speed,				// maximum speed (rad/s)
	min_speed_trip,			// clamp val for minimum speed (for trip detection)
	max_speed_trip,			// clamp val for maximum speed (for trip detection)
	min_s_trip,				// minimum speed (for trip detection)
	max_s_trip,				// maximum speed (for trip detection)
	Speed_est,				// estimated speed
	omrif,					// reference speed
	omrif_abs,				// |reference speed|
	f_omrif,				// reference speed (filtered)
	omegae,					// electrical angular speed
	omf,					// electrical angular speed (filtered)
	maxerr,					// maximum speed error (rad/s)
	amp_i,					// current vector amplitude (derived from iam, ibm)
	//arg_i,					// current vector phase (derived from iam, ibm)
	amp_v,					// voltage vector amplitude (derived from vam, vbm)
	//arg_v,					// voltage vector phase (derived from vam, vbm)
	f_amp_i,				// filtered current vector amplitude (derived from idmf, iqmf)
	f_arg_i,				// filtered current vector phase (derived from idmf, iqmf)
	f_amp_v,				// filtered voltage vector amplitude (derived from vdmf, vqmf)
	f_arg_v,				// filtered voltage vector phase (derived from vdmf, vqmf)
	delta_ph,				// phase variation (during startup)
	SystemPhase,			// imposed electrical phase
	Phase_est,				// estimated electrical phase
	Phase_offset,			// optional phase offset
	FluxAmp,				// flux amplitude estimation
	FluxPhase;				// first estimation of phase (not compensated)

#ifdef CPI_TUN
// current PI tuning
#define I_PIT_RATIO1	0.8f	// 80% of i_max
float
	ratio,
	cur_ampf;
unsigned short
	pit_s,
	pit_c;
#endif	// CPI_TUN

#ifdef CPI_AUT
// automatic current PI tuning
#define I_PIT_RATIO2	0.8f	// 80% of i_max
#define R_UP			0
#define R_MS			1
#define R_DW			2
#define R_ST			3
#define R_EN			4
float
	v_req;
unsigned short
	pitun_status;
#endif	// CPI_AUT

// motor parameters identification
#ifdef MOT_IDENT
#define IDENT_SPEED_HZ	20.0f
#define X_TIM1			CNT_MS( ID_TIM1_MS )	// current rising + alignment + stabilization
#define X_TIM2			CNT_MS( ID_TIM2_MS )	// first measurement step
#define X_TIM3			CNT_MS( ID_TIM3_MS )	// current falling + stabilization
#define X_TIM4			CNT_MS( ID_TIM4_MS )	// second measurement step
#define X_TIM5			CNT_MS( ID_TIM5_MS )	// speed rising + stabilization
#define X_TIM6			CNT_MS( ID_TIM6_MS )	// third measurement step
#define X_TIM7			CNT_MS( ID_TIM7_MS )	// current falling + stabilization
#define X_TIM8			CNT_MS( ID_TIM8_MS )	// fourth measurement step
#define INI_IDENT		{x_cnt=0; MC_Ident_IniVar(SAM_FRE_HZ);}
#define IDENT_ENDED		(x_cnt>X_TIM8)
signed long
	x_cnt=0;			// identification synchronization counter
float
	iden_rs,
	iden_ls,
	iden_fl,
	om_ident,			// identification speed
	om_r,				// idetification speed ramp
	ial_1,				// identification current 1
	ial_2,				// identification current 2
	ial_3,				// identification current 3
	ial_r;				// identification current ramp
#endif	// MOT_IDENT

// oscilloscope window in GUI
#ifdef OSC_WIN
float
	phmem;
signed short
	sample,
	bcntm,
	bcnt0,
	bcnt1;
//signed long
//	samacc;
#endif	// OSC_WIN

// current values used in inverter non linearities compensation
#ifdef DTC_REF
float
	iur,				// u current reference
	ivr,				// v current reference
	iwr;				// w current reference
#define IU				iur
#define IV				ivr
#define IW				iwr
#else	// DTC_REF
#define IU				ium
#define IV				ivm
#define IW				iwm
#endif	// DTC_REF

// inverter non linearities compensation
#ifdef INLCOMP

#define T_INL_DELTAIN	((float)(T_INL_INMAX/T_INL_INDMAX))
#define T_INL_UDELTAIN	((float)(T_INL_INDMAX/T_INL_INMAX))
unsigned short
	inl_ind;
float
	eu,
	ev,
	ew;

#ifndef INL_DET

// inverter non linearities compensation tables
const float tab_inlo[2]={
	0.0f,
	((float)INL_O)};
const float tab_inlk[T_INL_INDMAX+1]={
	0.0f,
	((float)(INL_01*PWM_FRE_HZ/1000000.0)),
	((float)(INL_02*PWM_FRE_HZ/1000000.0)),
	((float)(INL_03*PWM_FRE_HZ/1000000.0)),
	((float)(INL_04*PWM_FRE_HZ/1000000.0)),
	((float)(INL_05*PWM_FRE_HZ/1000000.0)),
	((float)(INL_06*PWM_FRE_HZ/1000000.0)),
	((float)(INL_07*PWM_FRE_HZ/1000000.0)),
	((float)(INL_08*PWM_FRE_HZ/1000000.0)),
	((float)(INL_09*PWM_FRE_HZ/1000000.0)),
	((float)(INL_10*PWM_FRE_HZ/1000000.0)),
	((float)(INL_11*PWM_FRE_HZ/1000000.0)),
	((float)(INL_12*PWM_FRE_HZ/1000000.0)),
	((float)(INL_13*PWM_FRE_HZ/1000000.0)),
	((float)(INL_14*PWM_FRE_HZ/1000000.0)),
	((float)(INL_15*PWM_FRE_HZ/1000000.0)),
	((float)(INL_16*PWM_FRE_HZ/1000000.0)),
	((float)(INL_17*PWM_FRE_HZ/1000000.0)),
	((float)(INL_18*PWM_FRE_HZ/1000000.0)),
	((float)(INL_19*PWM_FRE_HZ/1000000.0)),
	((float)(INL_20*PWM_FRE_HZ/1000000.0)),
	((float)(INL_21*PWM_FRE_HZ/1000000.0)),
	((float)(INL_22*PWM_FRE_HZ/1000000.0)),
	((float)(INL_23*PWM_FRE_HZ/1000000.0)),
	((float)(INL_24*PWM_FRE_HZ/1000000.0)),
	((float)(INL_25*PWM_FRE_HZ/1000000.0)),
	((float)(INL_26*PWM_FRE_HZ/1000000.0)),
	((float)(INL_27*PWM_FRE_HZ/1000000.0)),
	((float)(INL_28*PWM_FRE_HZ/1000000.0)),
	((float)(INL_29*PWM_FRE_HZ/1000000.0)),
	((float)(INL_30*PWM_FRE_HZ/1000000.0)),
	((float)(INL_31*PWM_FRE_HZ/1000000.0))};
#define MAX_VERR		((float)(tab_inlo[1]+vbus*tab_inlk[31]))
#define MC_INLO( X )	(MC_TabCal(( X ), T_INL_UDELTAIN, ((float *)tab_inlo), 1))
#define MC_INLK( X )	(MC_TabCal(( X ), T_INL_UDELTAIN, ((float *)tab_inlk), T_INL_INDMAX))
#define MC_INLU( X )	((MC_INLO( X )+vbus*MC_INLK( X )))
#define MC_INL( X )		(( X )>=0.0f? MC_INLU( X ): (-(MC_INLU(-( X )))))

#else	// INL_DET

unsigned short
	inl_sta,
	inl_cnt;
float
	inl_rif,
	v_v[T_INL_INDMAX+1],
	i_v[T_INL_INDMAX+1],
	b_v[T_INL_INDMAX+1],
	tab_inl[T_INL_INDMAX+1]= {
		0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f,
		0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f,
		0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f,
		0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f};
#define MAX_VERR		0.0f
#define MC_INLU( X )	(vbus*MC_TabCal(( X ), T_INL_UDELTAIN, ((float *)tab_inl), T_INL_INDMAX))
#define MC_INL( X )		(( X )>=0.0f? MC_INLU( X ): -(MC_INLU(-( X ))))
#define R_CUR			0
#define DO_M1			1
#define DO_M2			2
#define DO_M3			3
#define W_END			4
#define DI_MEAS			0.5f
#define I_MEAS2			(T_INL_INMAX+0.2f)
#define I_MEAS1			(I_MEAS2+DI_MEAS)
#define IM_RAMP			((float)(2.0*I_MEAS1/SAM_FRE_HZ))	// 2.0sec ramp time
#endif	// INL_DET

#endif	// INLCOMP

#ifdef MACRO_DEBUG
float
	p_pwm_period_s,			// pwm period in sec
	p_sam_period_s,			// sampling period in sec
	p_mtu_fre_hz,			// MTU frequency in Hz
	p_pwm_per,				// MTU pulses in 1 pwm period
	p_sam_per,				// MTU pulses in 1 sampling period
	p_sam_per_s,			// actual sampling period in sec
	p_pwm_fre_hz,			// actual pwm frequency in Hz
	p_sam_fre_hz,			// actual sampling frequency in Hz
	p_num_int,				// main loop timing
	p_mloop_per_s,			// actual main loop time in sec
	p_mloop_fre_hz,			// main loop frequency in Hz
	p_semiper,				// MTU pulses in 1/2 pwm period
	p_deadtim,				// deadtimes
	p_semiperdead,			// half period plus deadtimes
	p_kfo,					// Omega[rad/sec]=KFO*Freq[Hertz]
	p_kof,					// Freq[Hertz]=KOF*Omega[rad/sec]
	p_krpmo,				// Omega[rad/sec]=KRPMO*velocita[rpm]
	p_korpm,				// speed[rpm]=KORPM*Omega[rad/sec]
	p_kadi,					// current A/D conversion constant
	p_kadv;					// voltage A/D conversion constant
void macro_debug(void)
{
	p_pwm_period_s=PWM_PERIOD_S;
	p_sam_period_s=SAM_PERIOD_S;
	p_mtu_fre_hz=MTU_FRE_HZ;
	p_pwm_per=PWM_PER;
	p_sam_per=SAM_PER;
	p_pwm_fre_hz=PWM_FRE_HZ;
	p_sam_fre_hz=SAM_FRE_HZ;
	p_sam_per_s=SAM_PER_S;
	p_num_int=NUM_INT;
	p_mloop_per_s=MLOOP_PER_S;
	p_mloop_fre_hz=MLOOP_FRE_HZ;
	p_semiper=SEMIPER;
	p_deadtim=DEADTIM;
	p_semiperdead=SEMIPERDEAD;
	p_kfo=KFO;
	p_kof=KOF;
	p_krpmo=KRPMO;
	p_korpm=KORPM;
	p_kadi=KADI;
	p_kadv=KADV;
}
#endif	// MACRO_DEBUG

// timing macro
#define	US10		167U	// constant corresponding to 0.01 ms @100MHz

void MC_Delay(unsigned long time)
/*******************************************************************************
*	Description:	delay
*******************************************************************************/
{
	u32	i;

	for(; time>0; --time)
		for(i=US10; i>0; --i)
			nop();
}

unsigned char MC_WaitSync(void)
/***************************************************************************
*	Description:	time synchronization control function
***************************************************************************/
{
	if(cnt_int)
		return(1);
	cnt_int=NUM_INT;
	return(0);
}

float MC_TabCal(float in, float udeltain, float *table, unsigned short indmax)
/*******************************************************************************
*	Description:	linear interpolation from a table
*******************************************************************************/
{
	float			af;
	unsigned short	index;

	if(in<=0.0f)
		return(table[0]);
	af=in*udeltain;
	index=af;
	if(index>=indmax)
		return(table[indmax]);
	af-=index;
	return(table[index]+af*(table[index+1]-table[index]));
}

// debug PWM outputs
#define DBPWM_SH		8
#define DBPWM_PER		258	// 2^DBPWM_SH+2

void MC_StartDBPWM(void)
/***************************************************************************
*	Description:	set and start debug pwm outputs
*	Note:			debug pwm outputs can be used if the corresponding timer
*					is not used for other functions
***************************************************************************/
{
//  35 PB0 MTIOC0D MOSI-B					DIG OUT MTIOC0D_UP35	DEB_PWM1
//  36 PA5 ADTRG1#-A MTIOC1A MISO-B			DIG OUT MTIOC1A_UP36	DEB_PWM2
//  37 PA4 ADTRG0#-A MTIOC1B RSPCK-B		DIG OUT MTIOC1B_UP37	DEB_PWM3

	// wake up module before accessing registers
	//WAKEUP_MTU
	//MC_Delay(10);

	// stop channel 0, 1 and clear timers
	MTU.TSTRA.BIT.CST0=0;
	MTU.TSTRA.BIT.CST1=0;
	MTU0.TCNT=0;
	MTU1.TCNT=0;

	/* select pwm mode 2 for MTU0, MTU1 */
	MTU0.TMDR1.BYTE=0x03;
	//MTU0.TMDR1.BIT.BFE=0;		// reserved in TMDR0
	//MTU0.TMDR1.BIT.BFB=0;		// no buffer
	//MTU0.TMDR1.BIT.BFA=0;		// no buffer
	//MTU0.TMDR1.BIT.MD=3;		// select pwm mode 2
	MTU1.TMDR1.BYTE=0x03;
	//MTU1.TMDR1.BIT.MD=3;		// select pwm mode 2 (no buffer possible)

	/* select channel 0 for counter clock, select TGRA0 as period register */
	MTU0.TCR.BYTE=0xC0;
	//MTU0.TCR.BIT.CCLR=1;		// counter cleared by TGRA0 compare match
	//MTU0.TCR.BIT.CKEG=0;		// count at rising edge
	//MTU0.TCR.BIT.TPSC=0;		// no prescaler

	/* select synchronous operation for channels 0, 1 */
	MTU.TSYRA.BYTE=0x03;
	//MTU.TSYRA.BIT.SYNC4=0;
	//MTU.TSYRA.BIT.SYNC3=0;
	//MTU.TSYRA.BIT.SYNC2=0;
	//MTU.TSYRA.BIT.SYNC1=1;	// synchronous operation for channel 1
	//MTU.TSYRA.BIT.SYNC0=1;	// synchronous operation for channel 0

	/* select synchronous clearing for channel 1 */
	MTU1.TCR.BYTE=0x60;
	//MTU1.TCR.BIT.CCLR=3;		// synchronous clearig
	//MTU1.TCR.BIT.CKEG=0;		// count at rising edge
	//MTU1.TCR.BIT.TPSC=0;		// no prescaler

	/* select duty register (TIOC0D) */
	//MTU0.TIORL.BYTE=0x20;
	MTU0.TIORL.BIT.IOD=2;		// initial output 0, output 1 at compare match (no buffer possible)

	/* select duty registers (TIOC1A, TIOC1B) */
	MTU1.TIOR.BYTE=0x22;
	//MTU1.TIOR.BIT.IOB=2;		// initial output 0, output 1 at compare match (no buffer possible)
	//MTU1.TIOR.BIT.IOA=2;		// initial output 0, output 1 at compare match (no buffer possible)

	/* set period in TGRA0, duties in TGRD0, TGRA1, TGRB1 */
	MTU0.TGRA=DBPWM_PER;		// period
	MTU0.TGRD=DBPWM_PER+2;		// no compare match-> duty=0
	MTU1.TGRA=DBPWM_PER+2;		// no compare match-> duty=0
	MTU1.TGRB=DBPWM_PER+2;		// no compare match-> duty=0

	/* start timers */
	MTU.TSTRA.BYTE|=0x03;		// simultaneously start timer 0, 1
	//MTU.TSTRA.BIT.CST0=1;
	//MTU.TSTRA.BIT.CST1=1;
}

void MC_SetDBPWM1_TP7(unsigned short val)
/***************************************************************************
*	Description:	set debug pwm output duty (no buffered operation!)
*					TP7 is the filtered output of MTIOC0D (DEB_PWM1)
***************************************************************************/
{
//  35 PB0 MTIOC0D MOSI-B				DIG OUT MTIOC0D_UP35	DEB_PWM1
	unsigned short	aus;

	aus=val>>(16-DBPWM_SH);
	aus+=1;
	aus=DBPWM_PER-aus;
	MTU0.TGRD=aus;
}

void MC_SetDBPWM2_TP5(unsigned short val)
/***************************************************************************
*	Description:	set debug pwm output duty (no buffered operation!)
*					TP5 is the filtered output of MTIOC1A (DEB_PWM2)
***************************************************************************/
{
//  36 PA5 ADTRG1#-A MTIOC1A MISO-B		DIG OUT MTIOC1A_UP36	DEB_PWM2
	unsigned short	aus;

	aus=val>>(16-DBPWM_SH);
	aus+=1;
	aus=DBPWM_PER-aus;
	MTU1.TGRA=aus;
}

void MC_SetDBPWM3_TP6(unsigned short val)
/***************************************************************************
*	Description:	set debug pwm output duty (no buffered operation!)
*					TP6 is the filtered output of MTIOC1B (DEB_PWM3)
***************************************************************************/
{
//  37 PA4 ADTRG0#-A MTIOC1B RSPCK-B	DIG OUT MTIOC1B_UP37	DEB_PWM3
	unsigned short	aus;

	aus=val>>(16-DBPWM_SH);
	aus+=1;
	aus=DBPWM_PER-aus;
	MTU1.TGRB=aus;
}
// end of debug PWM outputs

// current reading functions
void MC_SetOff(void)
/***************************************************************************
*	Description:	current offsets measurement
***************************************************************************/
{
	unsigned long i, of0, of1, of2, ref;

	// wake up modules before accessing registers
	WAKEUP_S12AD
	WAKEUP_AD
	MC_Delay(100);
	INI_S12AD0						// single conversion
	INI_AD0

	// measurement cycle
	for(i=0, of0=0, of1=0, of2=0, ref=0; i<32; i++) {
		S12AD0.ADANS.BIT.CH=0;		// AN000_IW
		S12AD0_START
		WAIT_S12AD0
		of0+=AN000_IW;
		MC_Delay(10);
		S12AD0.ADANS.BIT.CH=1;		// AN001_IV
		S12AD0_START
		WAIT_S12AD0
		of1+=AN001_IV;
		MC_Delay(10);
		S12AD0.ADANS.BIT.CH=2;		// AN002_IU
		S12AD0_START
		WAIT_S12AD0
		of2+=AN002_IU;
		MC_Delay(10);
		AD0.ADCSR.BIT.CH=0;			// AN0_REF
		AD0_START
		WAIT_AD0
		ref+=AN0_REF;
		MC_Delay(100);
	}
	ium_off=of2*((float)(1.0/32.0));
	ivm_off=of1*((float)(1.0/32.0));
	iwm_off=of0*((float)(1.0/32.0));
	vol_ref=ref*((float)(1.0/32.0));
	k_vref=VREF_AD/vol_ref;
	kadi=KADI*k_vref;
	kadv=KADV*k_vref;
}

void MC_ReadVref(void)
/***************************************************************************
*	Description:	voltage reference reading, filtering and A/D conversion
*					constants correction
***************************************************************************/
{
	vol_ref+=(AN0_REF-vol_ref)*VR_UKT;
	k_vref=VREF_AD/vol_ref;
	kadi=KADI*k_vref;
	kadv=KADV*k_vref;
}

void MC_OffRefresh(void)
/***************************************************************************
*	Description:	current offset refresh (in case of three shunts current
*					reading)
***************************************************************************/
{
	iwm_off+=(AN000_IW-iwm_off)*OFF_UKT;
	ivm_off+=(AN001_IV-ivm_off)*OFF_UKT;
	ium_off+=(AN002_IU-ium_off)*OFF_UKT;
}

void MC_ReadTS(float *u, float *v, float *w)
/***************************************************************************
*	Description:	three shunts conversion results reading;
*					the choice of the two valid results is based on the
*					global variable cr_ts;
*					conversion in ampere is also made
***************************************************************************/
{
	float afu, afv, afw;

	iwad=AN000_IW;
	ivad=AN001_IV;
	iuad=AN002_IU;
	switch(cr_ts) {
		case CR_UV:
			afu=kadi*(iuad-ium_off);
			afv=kadi*(ivad-ivm_off);
			afw=-afu-afv;
		break;
		case CR_WU:
			afw=kadi*(iwad-iwm_off);
			afu=kadi*(iuad-ium_off);
			afv=-afw-afu;
		break;
		default:	// case CR_VW:
			afv=kadi*(ivad-ivm_off);
			afw=kadi*(iwad-iwm_off);
			afu=-afv-afw;
		break;
	}
	*u=afu;
	*v=afv;
	*w=afw;
}
// end of current reading functions

// PWM management functions
void MC_IniPWM(void)
/***************************************************************************
*	Description:	pwm initialization
***************************************************************************/
{

	// wake up module before accessing registers
	WAKEUP_MTU
	MC_Delay(100);

	// I/O pins setting is already done in hwsetup.c

	// MTU general settings
	MTU.TRWERB.BIT.RWE=1;		// MTU registers writing enable
	MTU.TSTRB.BIT.CST6=0;		// stop timer 6
	MTU.TSTRB.BIT.CST7=0;		// stop timer 7

	MTU6.TCR.BYTE=0x00;			//
	//MTU6.TCR.BIT.CCLR=0;		// synchronous clearing
	//MTU6.TCR.BIT.CKEG=0;		// count at rising edge
	//MTU6.TCR.BIT.TPSC=0;		// no peripheral clock frequency divider
	MTU7.TCR.BYTE=0x00;			//
	//MTU7.TCR.BIT.CCLR=0;		// synchronous clearing
	//MTU7.TCR.BIT.CKEG=0;		// count at rising edge
	//MTU7.TCR.BIT.TPSC=0;		// no peripheral clock frequency divider

	// registers and buffers setting
	MTU6.TCNT=DEADTIM;			// deadtime in TCNT6
	MTU7.TCNT=0;				// 0 in TCNT7
	MTU.TDDRB=DEADTIM;			// deadtime in TDDR
	MTU.TCDRB=SEMIPER;			// half pwm period in TCDR (note: MUST be > (TDDR*2+2)
	MTU.TCBRB=SEMIPER;			// buffer for TCDR
	MTU6.TGRA=SEMIPERDEAD;		// semiper + deadtime in TGRA6
	MTU6.TGRB=SEMIPERDEAD;		//
	MTU7.TGRA=SEMIPERDEAD;		//
	MTU7.TGRB=SEMIPERDEAD;		//
	MTU6.TGRC=SEMIPERDEAD;		// buffer for TGRA6
	MTU6.TGRD=SEMIPERDEAD;		// buffer for TGRB6
	MTU7.TGRC=SEMIPERDEAD;		// buffer for TGRA7
	MTU7.TGRD=SEMIPERDEAD;		// buffer for TGRB7

	// pwm frequency output enable and active levels setting
	MTU.TOCR1B.BIT.PSYE=0;		// disable output of pwm frequency
	MTU.TOCR1B.BIT.OLSN=0;		// select low-active output level
	MTU.TOCR1B.BIT.OLSP=0;		// select low-active output level

	// buffer setting
	MTU6.TMDR1.BIT.BFB=1;		// TGRD6 is a buffer for TGRB6
	MTU6.TMDR1.BIT.BFA=1;		// TGRC6 is a buffer for TGRA6
	//MTU7.TMDR1.BIT.BFB=1;		// TGRD7 is a buffer for TGRB7
	//MTU7.TMDR1.BIT.BFA=1;		// TGRC7 is a buffer for TGRA7

	// complementary pwm mode setting
	MTU6.TMDR1.BIT.MD=0x0D;		// complementary PWM mode, transmit at crest
	//MTU6.TMDR1.BIT.MD=0x0E;	// complementary PWM mode, transmit at trough
	//MTU6.TMDR1.BIT.MD=0x0F;	// complementary PWM mode, transmit at crest and trough
	//MTU7.TMDR1.BIT.MD=0;		// do not set here, but only in MTU6.TMDR1

	// output disable
	MTU.TOERB.BYTE=0xC0;		// all outputs disabled (at inactive level)
	//MTU.TOERB.BYTE=0xFF;		// all outputs enabled

	// POE init for pwm pins protection
	IOPORT.PFMPOE.BIT.POE4E=1;	// B1	pin43, POE4 input pin is enabled (OVCURR)
	//PORT9.DR.BIT.B6=1;		// already done in hwsetup
	//PORT9.DDR.BIT.B6=0;		// already done in hwsetup
	//PORT9.ICR.BIT.B6=1;		// already done in hwsetup
	POE.ICSR2.BIT.POE4M=1;		// B1, B0  low level sampling 16 times @(PCLK/8) (2.56 us @50MHz)  (can be modied only once)
	POE.OCSR2.BIT.OCE2=1;		// B9 high impedance enable
	POE.POECR2.WORD|=0x0700;	// place pins in high impedance state if problems

}

void MC_StartPWM(void)
/***************************************************************************
*	Description:	start pwm and main interrupt (but not the outputs)
***************************************************************************/
{

	// interrupt skipping (Timer 7 underflow -> trough)
	//MTU.TITMRB.BIT.TITM=0;		// already ok
	MTU.TITCR1B.BYTE=0x00;			// Timer Interrupt Skipping Set Register 1B (Reset val 0x00)
	//MTU.TITCR1B.BIT.T6AEN=0;		// B7
	//MTU.TITCR1B.BIT.T6ACOR=0;		// B6, B5, B4
	//MTU.TITCR1B.BIT.T7VEN=0;		// B3
	//MTU.TITCR1B.BIT.T7VCOR=0;		// B2,B1, B0
	MTU.TITCR1B.BYTE|=INT_SKIP;

/*
	// A/D converter setting for automatic current conversions (3 shunts)
	MTU7.TIER.BIT.TTGE2=1;			// enable A/D trigger at TCNT7 underflow (trough)
	S12AD0.ADSTRGR.BIT.ADSTRS0=7;	// select TRGA7N (trough) as trigger source
	S12AD0.ADANS.BIT.CH=2;			// channel select AN100, AN101, AN102
	//S12AD0.ADCER.BIT.SHBYP=0;		// B0		dedicated S&H circuit is selected
	S12AD0.ADCSR.WORD=0x2E;			// reset 0x00
	//S12AD0.ADCSR.BIT.ADST=0;		// B7
	//S12AD0.ADCSR.BIT.ADCS=1;		// B6, B5	single cycle scan mode
	//S12AD0.ADCSR.BIT.ADIE=0;		// B4		interrupt enable
	//S12AD0.ADCSR.BIT.CKS=3;		// B3, B2	faster clock
	//S12AD0.ADCSR.BIT.TRGE=1;		// B1		enable MTU trigger
	//S12AD0.ADCSR.BIT.EXTRG=0;		// B0
*/
/*
	// interrupt setup (A/D end)
	S12AD0.ADCSR.BIT.ADIE=1;		// B4		interrupt enable
	ICU.IPR[0x48].BIT.IPR=3;		// interrupt priority (S12AD0.ADIE)
	ICU.IER[0x0C].BIT.IEN6=1;		// interrupt enable
*/

	// interrupt setup (TCNT7 underflow, trough)
	MTU7.TSR.BIT.TCFV=0;		// clear flag of TCNT7 underflow (trough)
	MTU7.TIER.BIT.TCIEV=1;		// enable interrupt request on TCFV flag
	ICU.IPR[0x60].BIT.IPR=3;	// interrupt priority (TCIV7 trough)
	ICU.IER[0x13].BIT.IEN1=1;	// interrupt enable

	// timer start
	MTU.TSTRB.BYTE|=0xC0;		// simultaneously start timer 6 and 7

	// POE interrupt enable
	if(POE.ICSR2.BIT.POE4F==1)
		POE.ICSR2.BIT.POE4F=0;	// reset flag
	POE.ICSR2.BIT.PIE2=1;		// interrupt request enable
	if(POE.OCSR2.BIT.OSF2==1)
		POE.OCSR2.BIT.OSF2=0;	// reset flag
	POE.OCSR2.BIT.OIE2=1;		// interrupt request enable
	ICU.IPR[0x67].BIT.IPR=8;	// interrupt priority
	ICU.IER[0x15].BIT.IEN3=1;	// interrupt enable

	// stop again output for safety
	MTU.TOERB.BYTE=0xC0;		// all outputs disabled (at inactive level)
	RES_COM_ON
	RES_COM_REQ
}

void MC_StopOut(void)
/***************************************************************************
*	Description:	disable pwm outputs
***************************************************************************/
{
#ifdef DEF_LOW_ON
	if(FAULT_ALL == UIF_R.var.all)
	{
		MTU.TOERB.BYTE = 0xC0;	// all outputs disabled (at inactive level)
	}
#else	// DEF_LOW_ON
	MTU.TOERB.BYTE = 0xC0;	// all outputs disabled (at inactive level)
#endif	// DEF_LOW_ON
	RES_COM_ON
	RES_COM_REQ
}

void MC_StartOut(void)
/***************************************************************************
*	Description:	enable pwm outputs inserting a loop for bootstrap
***************************************************************************/
{
	SET_COM_REQ
	if(0xFF == MTU.TOERB.BYTE)
	{
		SET_COM_ON
	}
	else
	{
		RES_COM_ON
		MTU.TOERB.BYTE = 0xFF;	// all outputs enabled
	}
}

/******************************************************************************
* Function Name: MC_PWMGen
* Description  : output duties calc, and three shunts current reading status
*				 determination
* Arguments    : none
* Return Value : none
* Note         : global variables vuc, vvc and vwc are the requested voltages,
*				 global variable vbus is the bus voltage, and global variables
*				 duty_u, duty_v, duty_w, cr_ts are set by the routine
******************************************************************************/
void MC_PWMGen(void)
{
	float			af; 
	signed long		asl;
	signed short	ass, del, d_min, d_max;
#ifdef ICOMP
	signed short	i_min, i_med, i_max, d_med, dmax_max;
#endif	// ICOMP

#define	VBMIN				5.0f
#define SH_NRM				14
#define DELMAX_AMP			(((signed long)DELMAX) << SH_NRM)

	if(VBMIN > vbus)
	{
		vuc = 0.0f;
		vvc = 0.0f;
		vwc = 0.0f;
		duty_u = SEMIPERDEAD;
		duty_v = SEMIPERDEAD;
		duty_w = SEMIPERDEAD;
		return;
	}

	af = (-((float)SEMIPER)) / vbus;
	duty_u = af * vuc;
	duty_v = af * vvc;
	duty_w = af * vwc;

	if(duty_u >= duty_v)
	{
		if(duty_v >= duty_w)
		{
			d_min = duty_w;
			cr_ts = CR_UV;		// w is the min -> we choose iu and iv
			d_max = duty_u;
#ifdef ICOMP
			d_med = duty_v;
			i_min = IW;
			i_med = IV;
			i_max = IU;
#endif	// ICOMP
		}
		else
		{
			d_min = duty_v;
			cr_ts = CR_WU;		// v is the min -> we choose iw and iu
#ifdef ICOMP
			i_min = IV;
#endif	// ICOMP
			if(duty_u >= duty_w)
			{
				d_max = duty_u;
#ifdef ICOMP
				d_med = duty_w;
				i_med = IW;
				i_max = IU;
#endif	// ICOMP
			}
			else
			{
				d_max = duty_w;
#ifdef ICOMP
				d_med = duty_u;
				i_med = IU;
				i_max = IW;
#endif	// ICOMP
			}
		}
	}
	else
	{
		if(duty_u >= duty_w)
		{
			d_min = duty_w;
			cr_ts = CR_UV;		// w is the min -> we choose iu and iv
			d_max = duty_v;
#ifdef ICOMP
			d_med = duty_u;
			i_min = IW;
			i_med = IU;
			i_max = IV;
#endif	// ICOMP
		}
		else {
			d_min = duty_u;
			cr_ts = CR_VW;		// u is the min -> we choose iv and iw
#ifdef ICOMP
			i_min = IU;
#endif	// ICOMP
			if(duty_v >= duty_w)
			{
				d_max = duty_v;
#ifdef ICOMP
				d_med = duty_w;
				i_med = IW;
				i_max = IV;
#endif	// ICOMP
			}
			else
			{
				d_max = duty_w;
#ifdef ICOMP
				d_med = duty_v;
				i_med = IV;
				i_max = IW;
#endif	// ICOMP
			}
		}
	}
	duty_u -= d_min;
	duty_v -= d_min;
	duty_w -= d_min;
	d_max -= d_min;

#ifdef ICOMP
	d_med -= d_min;
	if((i_max>0) || ((i_med>0) && (d_med > SEMIPER)))
	{
		dmax_max = SEMIPER;
	}
	else
	{
		dmax_max = (SEMIPER + DEADTIM);
	}
	del = dmax_max - MIN_TON_LOW;

	if(d_max > del)	// hexagonal saturation
	{
		asl = mul_w(del, (1 << SH_NRM));
		ass = div_w(asl, d_max);
#else	// ICOMP
	if(d_max > DELMAX)	// hexagonal saturation
	{
		ass = div_w(DELMAX_AMP, d_max);
#endif	// ICOMP
		asl = mul_w(ass, duty_u);
		duty_u = asl >> SH_NRM;
		asl = mul_w(ass, duty_v);
		duty_v = asl >> SH_NRM;
		asl = mul_w(ass, duty_w);
		duty_w = asl >> SH_NRM;
		duty_u += MIN_TON_LOW;
		duty_v += MIN_TON_LOW;
		duty_w += MIN_TON_LOW;
	}
	else
	{
#ifdef ICOMP
		del = dmax_max - d_max;
#else	// ICOMP
		del = SEMIPER - d_max;
#endif	// ICOMP
#ifdef CENTERED
		asl = mul_w((duty_u + duty_v + duty_w), ((1 << SH_NRM) / 3));
		ass = asl >> SH_NRM;
		if(PWM_CENTER > ass)
		{
			ass = PWM_CENTER - ass;
			if(ass < del)
			{
				del = ass;
			}
		}
#endif	// CENTERED
		duty_u += del;
		duty_v += del;
		duty_w += del;
	}
}
// end of PWM management functions

float MC_Vcomp(float amp, float cmp)
/***************************************************************************
*	Description:	calc of the component of a vector, given amplitude
*					and the other component (output >=0)
***************************************************************************/
{
	float p;

	p = (amp * amp) - (cmp * cmp);
	if(p > 0.0f)
		return(sqrtf(p));
	else
		return(0.0f);
}

void MC_IniPar(void)
/***************************************************************************
*	Description:	parameters initialization
***************************************************************************/
{
	unsigned short aus;

#ifdef MACRO_DEBUG
	macro_debug();
#endif	// MACRO_DEBUG

	// sampling frequency and pwm frequency informations setting
	UIF_R.var.pwmf=PWM_FRE_HZ;
	UIF_R.var.sf=SAM_FRE_HZ;

	// additional features enabling
#ifdef CPI_TUN
	ENA_CURPI_TUN
#endif	// CPI_TUN
#ifdef CPI_AUT
	ENA_CURPI_AUT
#endif	// CPI_AUT
#ifdef	MOT_IDENT
	ENA_AUTO_IDEN
#endif	//	MOT_IDENT
#ifdef OSC_WIN
	ENA_OSCI_WIND
#endif	// OSC_WIN

	// polar couples and related constants
	c_poli=C_POLI;
	krpmocp=c_poli*KRPMO;
	ukrpmocp=((float)(1.0f/krpmocp));
	maxerr=krpmocp*MAXERR_RPM;
#ifdef RAMP_CL0
	r_marg=R_MARG*krpmocp;
#endif	// RAMP_CL0

	// rpm max and min
	rpm_max=RPM_MAX;
	rpm_min=RPM_MIN;

	// max angular speed, max speed for trip detection
	max_speed=rpm_max*krpmocp;
	max_speed_trip=max_speed*T_MARGH;
	max_s_trip=max_speed_trip;

	// min angular speed, and min speed for trip detection
	min_speed=rpm_min*krpmocp;
	min_speed_trip=min_speed*T_MARGL;
	min_s_trip=min_speed_trip;

	// delta speed every startup step
	delta_om=min_speed*(1.0f/(CNT_MS(STUP_SPE_RT_MS)));	// to be added to speed at every step

	// acceleration ramp
	r_acc_ep=R_ACC;				// rpm/s
	r_acc=r_acc_ep*MLOOP_PER_S;	// rpm/one_loop_time

	// deceleration ramp
	r_dec_ep=R_DEC;				// rpm/s
	r_dec=r_dec_ep*MLOOP_PER_S;	// rpm/one_loop_time

	// startup current
	i_start_ep=I_START;
	i_start=i_start_ep*((float)(1.0/AMP_RES));
	is_rup=i_start*((float)(1.0/(CNT_MS(STUP_CUR_RT_MS))));
	is_rdw=i_start*((float)(1.0/(CNT_MS(STUP_CUR_FT_MS))));

	// maximum current
	i_max_ep=I_MAX;
	i_max=i_max_ep*((float)(1.0/AMP_RES));
	id_max=i_max*K_MARGI0;
#ifdef MOT_IDENT
	ial_1=i_max*HIG_CUR_RATIO;
	ial_2=i_max*MID_CUR_RATIO;
	ial_3=i_max*LOW_CUR_RATIO;
	ial_r=ial_1*(1.0f/(CNT_MS(CUR_RISE_TIME_MS)));
	om_ident=((float)(TWOPI*IDENT_SPEED_HZ));
	om_r=om_ident*(1.0f/(CNT_MS(SPE_RISE_TIME_MS)));
#endif	// MOT_IDENT

	// statoric resistence
	r_sta_ep=R_STA;
	r_sta=r_sta_ep*((float)(1.0/OHM_RES));

	// synchronous inductance
	l_syn_ep=L_SYN;
	l_syn=l_syn_ep*((float)(1.0/HEN_RES));

	// permanent magnets flux
	pm_flx_ep=PM_FLX;
	pm_flx=pm_flx_ep*((float)(1.0/WEB_RES));

	// flux amplitude feedback gain
	fb_gain_ep=FB_GAIN;
	fb_gain=fb_gain_ep;

	// PI gains
	kp_cur_ep=KP_CUR;
	kp_cur=kp_cur_ep*((float)(1.0/AMP_KPCUR));
	ki_cur_ep=KI_CUR;
	ki_cur=ki_cur_ep*((float)(1.0/(AMP_KICUR*SAM_FRE_HZ)));
	kp_vel_ep=KP_VEL;
	kp_vel=kp_vel_ep*((float)(1.0/AMP_KPVEL));
	ki_vel_ep=KI_VEL;
	ki_vel=ki_vel_ep*((float)(1.0/(AMP_KIVEL*SAM_FRE_HZ)));

	// flux and speed estimation functions init
#ifndef APP_INT
	McrpLibf_FluxEst_SetParam(r_sta, l_syn, pm_flx, fb_gain, SAM_PER_S, SPEED_LP_CUTOFF_HZ);
#else	// APP_INT
	McrpLibf_FluxEstA_SetParam(r_sta, l_syn, pm_flx, SAM_PER_S, SPEED_LP_CUTOFF_HZ);
#endif	// APP_INT

	// speed ramp state machine
	ramp_status=STOP_0;
	SET_STOPPED
	RES_STARTED

	// phase offset
	Phase_offset=0.0f;

	// buffers
	rbuf=((unsigned short *)(outbuf.ss));
	wbuf=((unsigned short *)(outbuf1.ss));
	pbuf=rbuf;
	for(aus=0; aus<OUTBUF_SDIM; aus++) {
		rbuf[aus]=0;
		wbuf[aus]=0;
	}

#ifdef OSC_WIN
	bcnt0=0;
	bcnt1=0;
	bcntm=0;
	phmem=0.0f;
	sample=0;
	RES_TRGED
	RES_RBUSY
	RES_CSELD
#endif	// OSC_WIN

	// mode
	SET_NORM_MODE

}

void MC_UpdPar(void)
/***************************************************************************
*	Description:	parameters update
***************************************************************************/
{
	// parameters
	if((rpm_max!=RPM_MAX) || (rpm_min!=RPM_MIN) || (c_poli!=C_POLI)) {
		rpm_max=RPM_MAX;
		rpm_min=RPM_MIN;
		c_poli=C_POLI;
		krpmocp=c_poli*((float) KRPMO);
		ukrpmocp=((float)(1.0f/krpmocp));
		maxerr=krpmocp*MAXERR_RPM;
		max_speed=rpm_max*krpmocp;
		min_speed=rpm_min*krpmocp;
		max_speed_trip=max_speed*T_MARGH;
		min_speed_trip=min_speed*T_MARGL;
		delta_om=min_speed*(1.0f/(CNT_MS(STUP_SPE_RT_MS)));	// to be added to speed at every step
#ifdef RAMP_CL0
		r_marg=R_MARG*krpmocp;
#endif	// RAMP_CL0
	}
	if(r_acc_ep!=R_ACC) {
		r_acc_ep=R_ACC;					// in rpm/sec
		r_acc=r_acc_ep*((float) MLOOP_PER_S);
	}
	if(r_dec_ep!=R_DEC) {
		r_dec_ep=R_DEC;
		r_dec=r_dec_ep*((float) MLOOP_PER_S);
	}
	if(i_start_ep!=I_START) {
		i_start_ep=I_START;
		i_start=i_start_ep*((float)(1.0/AMP_RES));
		is_rup=i_start*((float)(1.0/(CNT_MS(STUP_CUR_RT_MS))));
		is_rdw=i_start*((float)(1.0/(CNT_MS(STUP_CUR_FT_MS))));
	}
	if(i_max_ep!=I_MAX) {
		i_max_ep=I_MAX;
		i_max=i_max_ep*((float)(1.0/AMP_RES));
		id_max=i_max*K_MARGI0;
#ifdef MOT_IDENT
		ial_1=i_max*HIG_CUR_RATIO;
		ial_2=i_max*MID_CUR_RATIO;
		ial_3=i_max*LOW_CUR_RATIO;
		ial_r=ial_1*(1.0f/(CNT_MS(CUR_RISE_TIME_MS)));
#endif	// MOT_IDENT
	}
	if(r_sta_ep!=R_STA) {
		r_sta_ep=R_STA;
		r_sta=r_sta_ep*((float)(1.0/OHM_RES));
#ifndef APP_INT
		McrpLibf_FluxEst_SetParam(r_sta, l_syn, pm_flx, fb_gain, SAM_PER_S, SPEED_LP_CUTOFF_HZ);
#else	// APP_INT
		McrpLibf_FluxEstA_SetParam(r_sta, l_syn, pm_flx, SAM_PER_S, SPEED_LP_CUTOFF_HZ);
#endif	// APP_INT
	}
	if(l_syn_ep!=L_SYN) {
		l_syn_ep=L_SYN;
		l_syn=l_syn_ep*((float)(1.0/HEN_RES));
#ifndef APP_INT
		McrpLibf_FluxEst_SetParam(r_sta, l_syn, pm_flx, fb_gain, SAM_PER_S, SPEED_LP_CUTOFF_HZ);
#else	// APP_INT
		McrpLibf_FluxEstA_SetParam(r_sta, l_syn, pm_flx, SAM_PER_S, SPEED_LP_CUTOFF_HZ);
#endif	// APP_INT
	}
	if(pm_flx_ep!=PM_FLX) {
		pm_flx_ep=PM_FLX;
		pm_flx=pm_flx_ep*((float)(1.0/WEB_RES));
#ifndef APP_INT
		McrpLibf_FluxEst_SetParam(r_sta, l_syn, pm_flx, fb_gain, SAM_PER_S, SPEED_LP_CUTOFF_HZ);
#else	// APP_INT
		McrpLibf_FluxEstA_SetParam(r_sta, l_syn, pm_flx, SAM_PER_S, SPEED_LP_CUTOFF_HZ);
#endif	// APP_INT
	}
	if(fb_gain_ep!=FB_GAIN) {
		fb_gain_ep=FB_GAIN;
		fb_gain=fb_gain_ep;
#ifndef APP_INT
		McrpLibf_FluxEst_SetParam(r_sta, l_syn, pm_flx, fb_gain, SAM_PER_S, SPEED_LP_CUTOFF_HZ);
#endif	// APP_INT
	}
	if(kp_cur_ep!=KP_CUR) {
		kp_cur_ep=KP_CUR;
		kp_cur=kp_cur_ep*((float)(1.0/AMP_KPCUR));
	}
	if(ki_cur_ep!=KI_CUR) {
		ki_cur_ep=KI_CUR;
		ki_cur=ki_cur_ep*((float)(1.0/(AMP_KICUR*SAM_FRE_HZ)));
	}
	if(kp_vel_ep!=KP_VEL) {
		kp_vel_ep=KP_VEL;
		kp_vel=kp_vel_ep*((float)(1.0/AMP_KPVEL));
	}
	if(ki_vel_ep!=KI_VEL) {
		ki_vel_ep=KI_VEL;
		ki_vel=ki_vel_ep*((float)(1.0/(AMP_KIVEL*SAM_FRE_HZ)));
	}

	// phase offset
	if(omrif<0.0f)
		Phase_offset=PHA_OFF*(-FTWOPI/360.0f);
	else if(omrif>0.0f)
		Phase_offset=PHA_OFF*(FTWOPI/360.0f);
	else
		Phase_offset=0.0f;

}

void MC_ConFil(void)
/***************************************************************************
*	Description:	output of measured values
***************************************************************************/
{

	// speed reference
	UIF_R.var.rif=rpmrif_y;

	// mechanical speed
	mec_rpm=omf*ukrpmocp;
	UIF_R.var.rpm=mec_rpm;

	// imposed frequency
	freq=omf*((float) KOF);
	UIF_R.var.fre=freq*10.0f;		// visualization in Hz/10

	// direct current
	UIF_R.var.id=idmf*1000.0f;		// visualization in mA(peak)

	// quadrature current
	UIF_R.var.iq=iqmf*1000.0f;		// visualization in mA(peak)

	// direct voltage
	UIF_R.var.vd=vdmf*10.0f;		// visualization in V/10(peak, star)

	// quadrature voltage
	UIF_R.var.vq=vqmf*10.0f;		// visualization in V/10(peak, star)

	// bus voltage
	UIF_R.var.vb=vbusf*10.0f;		// visualization in V/10

	// total current
	McrpLibf_xy_rt(idmf, iqmf, &f_amp_i, &f_arg_i);
	UIF_R.var.toti=f_amp_i*1000.0f;	// visualization in mA(peak)

	// total voltage
	McrpLibf_xy_rt(vdmf, vqmf, &f_amp_v, &f_arg_v);
	UIF_R.var.totv=f_amp_v*10.0f;	// visualization in V/10(peak, star)

	// dummy
#ifndef APP_INT
	FluxAmp=McrpLibf_FluxEst_GetAmp();
#else	// APP_INT
	FluxAmp=McrpLibf_FluxEstA_GetAmp();
#endif	// APP_INT
	UIF_R.var.du0=FluxAmp*WEB_RES;
//	UIF_R.var.du0=((float)(360/TWOPI))*McrpLibf_AngleNrm(f_arg_v-f_arg_i);
}

float MC_SpeedPI(float wr, float wm, float imx, float *i)
/***************************************************************************
*	Description:	Speed PI control
***************************************************************************/
{
	float af=wr-wm;

#ifdef SPEEDPI_ERRCLAMP
	if(af>maxerr)
		af=maxerr;
	else if(af<-maxerr)
		af=-maxerr;
#endif	// SPEEDPI_ERRCLAMP

	return(McrpLibf_PIW(af, imx, -imx, kp_vel, ki_vel, i));
}

void MC_IdIqPI(	float ird, float imd, float irq, float imq, float vmx,
				float *avd, float *avq, float *iid, float *iiq)
/***************************************************************************
*	Description:	d, q currents PI control
***************************************************************************/
{
	float af;

	if(vmx<=0.0f) {
		*avd=0.0f;
		*avq=0.0f;
		*iid=0.0f;
		*iiq=0.0f;
		return;
	}

	// id control
	af=vmx*K_MARGV0;			// vmaxd
	(*avd)=McrpLibf_PIW((ird-imd), af, -af, kp_cur, ki_cur, iid);

	// iq control
	af=MC_Vcomp(vmx, (*avd));	// vmaxq
	(*avq)=McrpLibf_PIW((irq-imq), af, -af, kp_cur, ki_cur, iiq);
}

void MC_CurPI(float ir, float im, float vmx, float *av, float *ii)
/***************************************************************************
*	Description:	current PI control
***************************************************************************/
{
	float vmn=-vmx;

	if((*av)<vmn)
		(*av)=vmn;
	else if((*av)>vmx)
		(*av)=vmx;
	(*av)+=McrpLibf_PIW((ir-im), vmx-(*av), vmn-(*av), kp_cur, ki_cur, ii);
}

#ifdef CPI_TUN
void MC_CurPITun(void)
/***************************************************************************
*	Description:	creation of a current step, to manually tune the
*					current PIs
***************************************************************************/
{
	if(pit_c<125) {
		idr=i_max*ratio;
		iqr=0.0f;
		if(pit_c<50) {				// 50 samples	0, .., 49, and 50 interrupts
			rbuf[pit_c++]=1000.0f*idm;
			cur_ampf=0.0f;
			pit_s=0;
		}
		else if(pit_c<65) {			// 15 samples	50, .., 64, and 45 interrupts
			if(pit_s<3) {
				++pit_s;
				cur_ampf+=idm;
			}
			else {
				rbuf[pit_c++]=1000.0f*cur_ampf/pit_s;
				cur_ampf=0.0f;
				pit_s=0;
			}
		}
		else if(pit_c<80) {			// 15 samples	65, .., 79, and 135 interrupts
			if(pit_s<9) {
				++pit_s;
				cur_ampf+=idm;
			}
			else {
				rbuf[pit_c++]=1000.0f*cur_ampf/pit_s;
				cur_ampf=0.0f;
				pit_s=0;
			}
		}
		else if(pit_c<95) {			// 15 samples	80, .., 94, and 405 interrupts
			if(pit_s<27) {
				++pit_s;
				cur_ampf+=idm;
			}
			else {
				rbuf[pit_c++]=1000.0f*cur_ampf/pit_s;
				cur_ampf=0.0f;
				pit_s=0;
			}
		}
		else if(pit_c<110) {		// 15 samples	95, .., 109, and 1215 interrupts
			if(pit_s<81) {
				++pit_s;
				cur_ampf+=idm;
			}
			else {
				rbuf[pit_c++]=1000.0f*cur_ampf/pit_s;
				cur_ampf=0.0f;
				pit_s=0;
			}
		}
		else /*if(pit_c<125)*/ {	// 15 samples	110, .., 124, and 3645 interrupts
			if(pit_s<243) {
				++pit_s;
				cur_ampf+=idm;
			}
			else {
				rbuf[pit_c++]=1000.0f*cur_ampf/pit_s;
				cur_ampf=0.0f;
				pit_s=0;
			}
		}
	}		// total: 125 samples and 5495 interrupts (around 343ms @16kHz)
	else {	// pit_c>=125
		idr=0.0f;
		iqr=0.0f;
		if(pit_c<150)
			++pit_c;
	}
}
#endif	// CPI_TUN

#ifdef CPI_AUT
void MC_CurPIAuto(void)
/***************************************************************************
*	Description:	creation of a voltage profile, used to automatically
*					tune the current PIs
***************************************************************************/
{
	idr=0.0f;
	iqr=0.0f;
	PIT_SetCur(idm, iqm);
	switch(pitun_status) {
		case R_UP:		// voltage ramp up till current reaches rated value
			if(PIT_RampUp(&v_req))
				pitun_status=R_MS;
			break;
		case R_MS:		// wait some time, measuring the current level (filtered)
			if(PIT_Measure(&v_req))
				pitun_status=R_DW;
			break;
		case R_DW:		// voltage ramp down
			if(PIT_RampDown(&v_req))
				pitun_status=R_ST;
			break;
		case R_ST:		// voltage step
			if(PIT_VoltStep(&v_req))
				pitun_status=R_EN;
			break;		// finished
		case R_EN:
		default:
			v_req=0.0f;
			break;
	}
	vdc=v_req;
	vqc=0.0f;
}
#endif	// CPI_AUT

#ifdef MOT_IDENT
void MC_MotIdent(void)
/***************************************************************************
*	Description:	motor identification sequence
***************************************************************************/
{

	iqr=0.0f;
	MC_Ident_SetMea(idm, iqm, vdm, vqm, omegae);

	if(x_cnt<0)
		x_cnt=0;
	else if(x_cnt<=X_TIM1) {	// id ramp up
		if(idr<(ial_1-ial_r))
			idr+=ial_r;
		else
			idr=ial_1;
		omegae=0.0f;
		SystemPhase=0.0f;
	}
	else if(x_cnt<X_TIM2) {		// first measurement cycle
		idr=ial_1;
		omegae=0.0f;
		SystemPhase=0.0f;
		MC_Ident_UpdAcc();
	}
	else if(x_cnt==X_TIM2) {	// first measurement acknowledge
		idr=ial_1;
		omegae=0.0f;
		SystemPhase=0.0f;
		MC_Ident_Step1();
	}
	else if(x_cnt<=X_TIM3) {	// id ramp dw
		if(idr>(ial_2+ial_r))
			idr-=ial_r;
		else
			idr=ial_2;
		omegae=0.0f;
		SystemPhase=0.0f;
	}
	else if(x_cnt<X_TIM4) {		// second measurement cycle
		idr=ial_2;
		omegae=0.0f;
		SystemPhase=0.0f;
		MC_Ident_UpdAcc();
	}
	else if(x_cnt==X_TIM4) {	// second measurement acknowledge
		idr=ial_2;
		omegae=0.0f;
		SystemPhase=0.0f;
		MC_Ident_Step2();
	}
	else if(x_cnt<=X_TIM5) {	// speed ramp
		idr=ial_2;
		if(omegae<(om_ident-om_r))
			omegae+=om_r;
		else
			omegae=om_ident;
		SystemPhase=McrpLibf_AngleNrm(SystemPhase+omegae*((float) SAM_PER_S));
	}
	else if(x_cnt<X_TIM6) {		// third measurement cycle
		idr=ial_2;
		omegae=om_ident;
		SystemPhase=McrpLibf_AngleNrm(SystemPhase+omegae*((float) SAM_PER_S));
		MC_Ident_UpdAcc();
	}
	else if(x_cnt==X_TIM6) {	// third measurement acknowledge
		idr=ial_2;
		omegae=om_ident;
		SystemPhase=McrpLibf_AngleNrm(SystemPhase+omegae*((float) SAM_PER_S));
		MC_Ident_Step3();
	}
	else if(x_cnt<=X_TIM7) {	// id ramp dw
		if(idr>(ial_3+ial_r))
			idr-=ial_r;
		else
			idr=ial_3;
		omegae=om_ident;
		SystemPhase=McrpLibf_AngleNrm(SystemPhase+omegae*((float) SAM_PER_S));
	}
	else if(x_cnt<X_TIM8) {		// fourth measurement cycle
		idr=ial_3;
		omegae=om_ident;
		SystemPhase=McrpLibf_AngleNrm(SystemPhase+omegae*((float) SAM_PER_S));
		MC_Ident_UpdAcc();
	}
	else if(x_cnt==X_TIM8) {	// fourth measurement acknowledge
		idr=ial_3;
		omegae=om_ident;
		SystemPhase=McrpLibf_AngleNrm(SystemPhase+omegae*((float) SAM_PER_S));
		MC_Ident_Step4();
	}
	else {						// waiting end
		idr=0.0f;
		omegae=0.0f;
		SystemPhase=0.0f;
	}
	if(x_cnt<0x7FFFFFFF)
		++x_cnt;
}
#endif	// MOT_IDENT

#ifdef INLCOMP
#ifdef INL_DET
void MC_InlTun(void)
/***************************************************************************
*	Description:	creation of a current profile, used to automatically
*					detect the inverter non-linearities
***************************************************************************/
{
	INL_SetMea(amp_v, amp_i, vbus);
	switch(inl_sta) {
		case R_CUR:
			if(inl_rif<(I_MEAS1-IM_RAMP))
				inl_rif+=IM_RAMP;
			else {
				inl_rif=I_MEAS1;
				inl_cnt=0;
				inl_sta=DO_M1;
			}
			break;
		case DO_M1:
			if(inl_cnt<CNT_MS(200))
				++inl_cnt;
			else if(inl_cnt<CNT_MS(300)) {
				++inl_cnt;
				INL_UpdAcc();
			}
			else {
				INL_SetRes1();
				inl_rif=I_MEAS2;
				inl_cnt=0;
				inl_sta=DO_M2;
			}
			break;
		case DO_M2:
			if(inl_cnt<CNT_MS(200))
				++inl_cnt;
			else if(inl_cnt<CNT_MS(300)) {
				++inl_cnt;
				INL_UpdAcc();
			}
			else {
				INL_SetRes2();
				inl_cnt=0;
				inl_ind=T_INL_INDMAX;
				inl_rif=T_INL_INMAX;
				inl_sta=DO_M3;
			}
			break;
		case DO_M3:
			if(inl_cnt<CNT_MS(200))
				++inl_cnt;
			if(inl_cnt<CNT_MS(300)) {
				++inl_cnt;
				INL_UpdAcc();
			}
			else {
				INL_SetResi(v_v, i_v, b_v, inl_ind);
				if(inl_ind>1) {
					--inl_ind;
					inl_rif=inl_ind*T_INL_DELTAIN;
					inl_cnt=0;
				}
				else {
					inl_rif=0.0f;
					inl_sta=W_END;
				}
			}
			break;
		default:
			inl_rif=0.0f;
		break;
	}
	INL_SetRef(inl_rif, &idr, &iqr);
}
#endif	// INL_DET
#endif	// INLCOMP

void MC_WorkMan(void)
/***************************************************************************
*	Description:	working mode management routine
***************************************************************************/
{
	float 				af, af1;
	static signed short	rif_mem;

	// alarm management
	if(UIF_R.var.all!=0) {
		MC_StopOut();
		if(UIF_R.var.all==TRIP_ALL) {
			if(	(UIF_W.var.rif==0) ||
				((rif_mem<0) && (UIF_W.var.rif>0)) ||
				((rif_mem>0) && (UIF_W.var.rif<0)) ) {
				UIF_R.var.all=0;
				RES_ALRM_ON
			}
		}
		RES_STARTED
		SET_STOPPED
		rpmrif_x=0.0f;
		rpmrif_y=0.0f;
		omrif=0.0f;
		ramp_status=STOP_0;
		rif_mem=UIF_W.var.rif;
		SET_NORM_MODE		// on alarm, normal mode is reestablished
		RES_STA_BSY
		SET_END_NOK
		RES_GUI_TRI
		RES_GUI_STP
		return;
	}
	rif_mem=UIF_W.var.rif;

	// working mode requests management
	if(!COM_REQ) {	// all requests are taken into consideration only if COM_OFF
#ifdef CPI_AUT
		if(CURPI_AUT_REQ)
			SET_CURPI_AUT
		else
#endif	// CPI_AUT
#ifdef CPI_TUN
		if(CURPI_TUN_REQ)
			SET_CURPI_TUN
		else
#endif	// CPI_TUN
#ifdef MOT_IDENT
		if(AUTO_IDEN_REQ)
			SET_AUTO_IDEN
#endif	// MOT_IDENT
		if(!NORM_MODE) {
			RES_STARTED
			SET_STOPPED
			rpmrif_x=0.0f;
			rpmrif_y=0.0f;
			omrif=0.0f;
			ramp_status=STOP_0;
			if(NORM_MODE_REQ) {
				UIF_R.var.kpi=((unsigned short)KP_CUR);
				UIF_R.var.kii=((unsigned short)KI_CUR);
				UIF_R.var.rs=((unsigned short)R_STA);
				UIF_R.var.ls=((unsigned short)L_SYN);
				UIF_R.var.fl=((unsigned short)PM_FLX);
				RES_STA_BSY
				SET_NORM_MODE
			}
		}
	}

#ifdef CPI_AUT
	// Automatic current PI tuning mode
	if(CURPI_AUT) {
		if(!COM_REQ) {
			if(GUI_TRI) {
				SET_STA_BSY
				PIT_IniVar(i_max*I_PIT_RATIO2, vbusf, SAM_PER_S);
				v_req=0.0f;
				pitun_status=R_UP;
				MC_StartOut();
			}
		}
		else if(GUI_STP) {
			MC_StopOut();
			SET_END_NOK
			RES_STA_BSY
		}
		else if(pitun_status>=R_EN) {
			MC_StopOut();
			UIF_R.var.kpi=AMP_KPCUR*PIT_ReadKP();
			UIF_R.var.kii=(AMP_KICUR*SAM_FRE_HZ)*PIT_ReadKI();
			RES_END_NOK
			RES_STA_BSY
		}
		else
			MC_StartOut();
		RES_GUI_TRI
		RES_GUI_STP
		return;
	}
#endif	// CPI_AUT

#ifdef CPI_TUN
	// Manual current PI tuning mode (here normal eeprom parameters are used)
	if(CURPI_TUN) {
		if(!COM_REQ) {
			if(GUI_TRI) {
				SET_STA_BSY
				ratio=0.01f*UIF_W.var.cra;
				if(ratio<0.0f)
					ratio=0.0f;
				else if(ratio>1.0f)
					ratio=1.0f;
				rbuf=((unsigned short *)(outbuf.ss));
				wbuf=((unsigned short *)(outbuf1.ss));
				pbuf=rbuf;
				pit_c=0;
				MC_StartOut();
			}
		}
		else if(GUI_STP) {
			MC_StopOut();
			SET_END_NOK
			RES_STA_BSY
		}
		else if(pit_c>=130) {
			MC_StopOut();
			RES_END_NOK
			RES_STA_BSY
		}
		else
			MC_StartOut();
		RES_GUI_TRI
		RES_GUI_STP
		return;
	}
#endif	// CPI_TUN

#ifdef MOT_IDENT
	// Motor parameters auto identification mode
	if(AUTO_IDEN) {
		if(!COM_REQ) {
			if(GUI_TRI) {
				SET_STA_BSY
				INI_IDENT
				MC_StartOut();
			}
		}
		else if(GUI_STP) {
			MC_StopOut();
			SET_END_NOK
			RES_STA_BSY
		}
		else if(IDENT_ENDED) {
			MC_StopOut();
			MC_Ident_CalMotPar(&iden_rs, &iden_ls, &iden_fl);
			UIF_R.var.rs=((unsigned short)(iden_rs*((float)OHM_RES)));
			UIF_R.var.ls=((unsigned short)(iden_ls*((float)HEN_RES)));
			UIF_R.var.fl=((unsigned short)(iden_fl*((float)WEB_RES)));
			RES_END_NOK
			RES_STA_BSY
		}
		else
			MC_StartOut();
		RES_GUI_TRI
		RES_GUI_STP
		return;
	}
#endif	// MOT_IDENT

	// From here normal mode management

	// preliminary reference management
#ifdef	ROT_REV
	af=-UIF_W.var.rif;
#else	// ROT_REV
	af=UIF_W.var.rif;
#endif	// ROT_REV
	if(af>=rpm_min) {
		if(af>rpm_max)
			af=rpm_max;
	}
	else if(af<=-rpm_min) {
		if(af<-rpm_max)
			af=-rpm_max;
	}
	else {
		af=0.0f;
#ifdef DEBUG_OPENLOOP
		if(DEB_OPNLP) {
			MC_StopOut();
			rpmrif_x=0.0f;
			rpmrif_y=0.0f;
			omrif=0.0f;
			ramp_status=STOP_0;
			RES_STARTED
			SET_STOPPED
			return;
		}
#endif	// DEBUG_OPENLOOP
	}

#ifdef INLCOMP
#ifdef INL_DET
	if(INL_TUN) {	// inverter non linearities measurement
	unsigned short aus;
		if(!COM_REQ) {
			if(inl_sta==R_CUR) {
				for(aus=0; aus<=T_INL_INDMAX; aus++)
					tab_inl[aus]=0.0f;
				INL_IniVar(T_INL_INDMAX);
				inl_sta=R_CUR;
				inl_rif=0.0f;
				MC_StartOut();
			}
		}
		else if(inl_sta==W_END) {
			MC_StopOut();
			INL_SetTab(v_v, i_v, b_v, tab_inl);
		}
		else
			MC_StartOut();
		RES_GUI_TRI
		return;
	}
	else {
		inl_sta=R_CUR;
	}
#endif	// INL_DET
#endif	// INLCOMP

#ifdef DCCUR_SETTING
	// DC current vector setting
	if(DCCUR_SET) {
		rpmrif_x=0.0f;
		rpmrif_y=0.0f;
		omrif=0.0f;
		ramp_status=STOP_0;
		RES_STARTED
		SET_STOPPED
		if(af==0.0f)
			MC_StopOut();
		else
			MC_StartOut();
		return;
	}
#endif	// DCCUR_SETTING

	// ramp management state-machine
	switch(ramp_status) {
		case START_P:
			RES_STOPPED
			MC_StartOut();
			rpmrif_x=rpm_min;
			rpmrif_y=rpm_min;
			if(af<rpm_min) {
				ramp_status=STOP_W;
				if(af<=-rpm_min)
					rpmrif_x=-rpm_min;
				else
					rpmrif_x=0.0f;
				break;
			}
			else if(!STARTED)
				break;
			ramp_status=GO_P;
			// no break
		case GO_P:
			RES_STOPPED
			MC_StartOut();
			if(af<rpm_min) {
				if(rpmrif_y<=rpm_min) {
					rpmrif_y=0.0f;
					ramp_status=STOP_W;
					if(af<=-rpm_min)
						rpmrif_x=-rpm_min;
					else
						rpmrif_x=0.0f;
					break;
				}
				rpmrif_x=rpm_min;
			}
			else
				rpmrif_x=af;
			af=rpmrif_y;
			if(af<(rpmrif_x-r_acc)) {
				RES_DECEL
				af+=r_acc;
			}
			else if(af>(rpmrif_x+r_dec)) {
				SET_DECEL
				if(NOT_RAMPLIM)
					af-=r_dec;
			}
			else {
				RES_DECEL
				af=rpmrif_x;
			}
#ifdef RAMP_CL0			// ramp clamp if heavy speed error
			af1=mec_rpm+MAXERR_RPM;
			if((af>af1) && (af1>rpm_min))
				af=af1;
#endif	// RAMP_CL0
			rpmrif_y=af;
			break;
		case START_N:
			RES_STOPPED
			MC_StartOut();
			rpmrif_x=-rpm_min;
			rpmrif_y=-rpm_min;
			if(af>-rpm_min) {
				ramp_status=STOP_W;
				if(af>=rpm_min)
					rpmrif_x=rpm_min;
				else
					rpmrif_x=0.0f;
				break;
			}
			else if(!STARTED)
				break;
			ramp_status=GO_N;
			// no break
		case GO_N:
			RES_STOPPED
			MC_StartOut();
			if(af>-rpm_min) {
				if(rpmrif_y>=-rpm_min) {
					rpmrif_y=0.0f;
					ramp_status=STOP_W;
					if(af>=rpm_min)
						rpmrif_x=rpm_min;
					else
						rpmrif_x=0.0f;
					break;
				}
				af=-rpm_min;
			}
			rpmrif_x=af;
			af=rpmrif_y;
			if(af>(rpmrif_x+r_acc)) {
				RES_DECEL
				af-=r_acc;
			}
			else if(af<(rpmrif_x-r_dec)) {
				SET_DECEL
				if(NOT_RAMPLIM)
					af+=r_dec;
			}
			else {
				RES_DECEL
				af=rpmrif_x;
			}
#ifdef RAMP_CL0			// ramp clamp if heavy speed error
			af1=mec_rpm-MAXERR_RPM;
			if((af<af1) && (af1<-rpm_min))
				af=af1;
#endif	// RAMP_CL0
			rpmrif_y=af;
			break;
		case STOP_W:
			RES_STARTED
			rpmrif_y=0.0f;
			MC_StartOut();
			if(af>=rpm_min) {
				rpmrif_x=rpm_min;
				if(STOPPED)
					ramp_status=START_P;
			}
			else if(af<=-rpm_min) {
				rpmrif_x=-rpm_min;
				if(STOPPED)
					ramp_status=START_N;
			}
			else {
				rpmrif_x=0.0f;
				if(STOPPED)
					ramp_status=STOP_0;
			}
			break;
		case STOP_0:
		default:
			SET_STOPPED
			RES_STARTED
			rpmrif_x=0.0f;
			rpmrif_y=0.0f;
			MC_StopOut();
			if(af>=rpm_min)
				ramp_status=START_P;
			else if(af<=-rpm_min)
				ramp_status=START_N;
			else
				ramp_status=STOP_0;
			break;
	}

	// ramp output update and conversion in rad/sec
	omrif=rpmrif_y*krpmocp;
	rpmrif_y_abs=(rpmrif_y<0.0f? -rpmrif_y: rpmrif_y);
	omrif_abs=rpmrif_y_abs*krpmocp;

	// trip limit values calc
#ifdef RAMP_CL0
	af1=omrif_abs*T_MARGL;
	af=omrif_abs-maxerr-r_marg;
	if(af>af1)
		af=af1;
	if(af<min_speed_trip)
		min_s_trip=min_speed_trip;
	else
		min_s_trip=af;
	af1=omrif_abs*T_MARGH;
	af=omrif_abs+maxerr+r_marg;
	if(af<af1)
		af=af1;
	if(af>max_speed_trip)
		max_s_trip=max_speed_trip;
	else
		max_s_trip=af;
#endif	// RAMP_CL0

}

#ifdef TRIP_CONTROL
void MC_TripCon(void)
/***************************************************************************
*	Description:	loosing of control of the phase estimation algorithm
*					check
***************************************************************************/
{
	if(startup_cnt>2) {
		if(ramp_status==GO_P) {
			if((Speed_est<min_s_trip) || (Speed_est>max_s_trip))
				trip_cnt+=1;
			else if(trip_cnt>=2)
				trip_cnt-=2;
			else
				trip_cnt=0;
		}
		else if(ramp_status==GO_N) {
			if((Speed_est>(-min_s_trip)) || (Speed_est<(-max_s_trip)))
				trip_cnt+=1;
			else if(trip_cnt>=2)
				trip_cnt-=2;
			else
				trip_cnt=0;
		}
		if(trip_cnt>=MAX_TRIP) {
			MC_StopOut();
			UIF_R.var.all=TRIP_ALL;
			SET_ALRM_ON
			trip_cnt=0;
		}
	}
	else
		trip_cnt=0;
}
#endif	// TRIP_CONTROL

#ifdef FLUXWEAK		// Flux weakening
void MC_FluxWeak(float vmf, float imax, float iref, float *i)
/*******************************************************************************
*	Description:	d reference calc based on voltage request
*******************************************************************************/
{
	if((vbus>VBUS_LIML) || (vmf<0.0f)) {
		if((*i)>-imax)
			(*i)-=INTID_RAMP;
	}
	else if((*i)>(iref+INTID_RAMP))
		(*i)-=INTID_RAMP;
	else if((*i)<(iref-INTID_RAMP))
		(*i)+=INTID_RAMP;
	else
		(*i)=iref;
}
#endif	// FLUXWEAK

void MC_ConInt(void)
/***************************************************************************
*	Description:	control interrupt
***************************************************************************/
{
	float			af;
	unsigned short	aus;

	// interrupt pending flag reset
	MTU7.TSR.BIT.TCFV=0;		// clear flag of TCNT7 underflow (trough)

	// timing debug
	SET_DL3

	// start VREF A/D conversion
	AD0.ADCSR.BIT.CH=0;		// AN0_REF
	AD0_START

	// start currents A/D conversion (here we are around 1us after PWM center)
	S12AD0.ADANS.BIT.CH=2;		// channel select AN000, AN001, AN002
	S12AD0.ADCSR.BYTE=0x2C;		// reset 0x00
	S12AD0_START

	// read VREF A/D conversion
	WAIT_AD0
	MC_ReadVref();

	// three shunts A/D conversion read
	WAIT_S12AD0
#ifndef DEF_LOW_ON
	if(COM_ON)
		MC_ReadTS(&ium, &ivm, &iwm);
	else {	// current offsets refresh
		MC_OffRefresh();
		ium=0.0f;
		ivm=0.0f;
		iwm=0.0f;
	}
#else	// DEF_LOW_ON
	MC_ReadTS(&ium, &ivm, &iwm);
#endif	// DEF_LOW_ON

	// start vbus A/D conversion
	S12AD0.ADCSR.BYTE=0x0C;
	S12AD0.ADANS.BIT.CH=3;
	S12AD0_START

	if(COM_ON) {
		// current transformations (previous angle)
		McrpLibf_uvw_alphabeta(ium, ivm, iwm, &iam, &ibm);
		McrpLibf_alphabeta_dq(iam, ibm, &idm, &iqm);

		// voltage measurements (related to currents measurements)
		vam=vac;
		vbm=vbc;
		vdm=vdc;
		vqm=vqc;
	}	// if(COM_ON)
	else {
		iam=0.0f;
		ibm=0.0f;
		idm=0.0f;
		iqm=0.0f;
		vam=0.0f;
		vbm=0.0f;
		vdm=0.0f;
		vqm=0.0f;
	}

	// read vbus measurement
	WAIT_S12AD0
	vbus_ad=AN003_VB;
	vbus=kadv*vbus_ad;					// dc bus voltage in Volt

	// angle management
	McrpLibf_AngleSet(SystemPhase);		// from here new angle (sin and cos) (conversions etc.)

	// voltages references and duty cycles calc
	if(COM_ON) {

#ifdef CPI_AUT
		if(CURPI_AUT) {
			MC_CurPIAuto();	// produces directly voltages references vdc, vqc
		}		// if(CURPI_AUT)
		else
#endif	// CPI_AUT
		{	// normal case: idr, iqr come from previous cycle
/*
			vdc=vqc=0.0f;
			// here you could insert decoupling, feed forward, etc.
			vdc=r_sta*idr-omegae*l_syn*iqr;
			vqc=r_sta*iqr+omegae*l_syn*idr;
			vdmax=vfmax*K_MARGV0;
			MC_CurPI(idr, idm, vdmax, &vdc, &idint);
			vqmax=MC_Vcomp(vfmax, vdc);
			MC_CurPI(iqr, iqm, vqmax, &vqc, &iqint);
*/
			// current PI controls
			MC_IdIqPI(idr, idm, iqr, iqm, vfmax, &vdc, &vqc, &idint, &iqint);
		}	// normal case

		McrpLibf_dq_alphabeta(vdc, vqc, &vac, &vbc);
		McrpLibf_alphabeta_uv(vac, vbc, &vuc, &vvc);
		vwc=-vuc-vvc;
#ifdef DTC_REF
		McrpLibf_dq_alphabeta(idr, iqr, &iar, &ibr);
		McrpLibf_alphabeta_uv(iar, ibr, &iur, &ivr);
		iwr=-iur-ivr;
#endif	// DTC_REF
#ifdef INLCOMP
		eu=MC_INL(IU);
		ev=MC_INL(IV);
		ew=MC_INL(IW);
		vuc+=eu;
		vvc+=ev;
		vwc+=ew;
#endif	// INLCOMP

		// pwm output generation
		MC_PWMGen(); // uses vuc, vvc, vwc to calculate duty_u, duty_v, duty_w

	}
	else {	// else of if(COM_ON)
		idint=0.0f;
		iqint=0.0f;
		iaint=0.0f;
		ibint=0.0f;
		vdc=0.0f;
		vqc=0.0f;
		vac=0.0f;
		vbc=0.0f;
		vuc=0.0f;
		vvc=0.0f;
		vwc=0.0f;
		duty_u=SEMIPERDEAD;	// if outputs enabled, this means the lower arm always on (used for BOOTSTRAP)
		duty_v=SEMIPERDEAD;	// if outputs enabled, this means the lower arm always on (used for BOOTSTRAP)
		duty_w=SEMIPERDEAD;	// if outputs enabled, this means the lower arm always on (used for BOOTSTRAP)
#ifdef DTC_REF
		iar=0.0f;
		ibr=0.0f;
		iur=0.0f;
		ivr=0.0f;
		iwr=0.0f;
#endif	// DTC_REF
#ifdef INLCOMP
		eu=0.0f;
		ev=0.0f;
		ew=0.0f;
#endif	// INLCOMP
	}

	// pwm output registers setting
	BUF_U=duty_u;
	BUF_V=duty_v;
	BUF_W=duty_w;

	// bus voltage filter (visualization)
	vbusf+=(vbus-vbusf)*VISU_UKT;					// DC bus voltage

	// bus overvoltage control
#ifdef VB_RAMP
	if(DECEL) {
		if(vbus>=VBUS_LIMH)
			SET_RAMPLIM
		else if(vbus<VBUS_LIML)
			RES_RAMPLIM
	}
	else
		RES_RAMPLIM
#endif	// VB_RAMP

	// bus voltage filter and vfmax calc
	xvb=vbus*KVOUT;							// effective voltage ratio
#ifdef INLCOMP
	if(xvb>(2*MAX_VERR))
		xvb-=(2*MAX_VERR);
	else
		xvb=0.0f;
#endif	// INLCOMP
	if(xvb<xvbf)
		xvbf+=(xvb-xvbf)*VB_UKT_FAST; 		// fast decrease
	else if(xvb>xvbf)
		xvbf+=(xvb-xvbf)*VB_UKT_SLOW;		// slow increase
	if(xvbf>0.0f) {
#ifdef OVERMOD
		// given VB, the max star vector is VB*(2/3) (with hexagonal saturation)
		vfmax=xvbf*((float) TWOTHIRDS);
#else	// OVERMOD
		// given VB, the max star vector is VB/SQRT3 (without hexagonal saturation)
		vfmax=xvbf*((float) SQRT3D3);
#endif	// OVERMOD
	}
	else {
		xvbf=0.0f;
		vfmax=0.0f;
	}

	if(COM_ON) {

		// effective output voltages re-calculation after duty cycles clamping in modulation
		af=vbus*((float)(-1.0/SEMIPER));
		vuc=af*duty_u;	// no problems regarding common offset
		vvc=af*duty_v;	// no problems regarding common offset
		vwc=af*duty_w;	// no problems regarding common offset
#ifdef INLCOMP
		vuc-=eu;
		vvc-=ev;
		vwc-=ew;
#endif	// INLCOMP
		McrpLibf_uvw_alphabeta(vuc, vvc, vwc, &vac, &vbc);
		McrpLibf_alphabeta_dq(vac, vbc, &vdc, &vqc);

		// measured current and voltages vectors amplitude
		//McrpLibf_xy_rt(iam, ibm, &amp_i, &arg_i);
		//McrpLibf_xy_rt(vam, vbm, &amp_v, &arg_v);
		amp_i=sqrtf(iam*iam+ibm*ibm);
		amp_v=sqrtf(vam*vam+vbm*vbm);

		// From here, current references calculation, depending on working mode

#ifdef CPI_TUN
		if(CURPI_TUN) {
			MC_CurPITun();
		}
		else
#endif	// CPI_TUN

#ifdef MOT_IDENT
		if(AUTO_IDEN) {
			MC_MotIdent();
		}
		else
#endif	// MOT_IDENT

#ifdef INLCOMP
#ifdef INL_DET
		if(INL_TUN) {
			MC_InlTun();
		}
		else
#endif	// INL_DET
#endif	// INLCOMP

#ifdef DCCUR_SETTING
		if(DCCUR_SET) {
			af=McrpLibf_AngleNrm(PHA_OFF*((float)(TWOPI/360)));
			McrpLibf_rt_xy(i_start, af, &idr, &iqr);
		}
		else
#endif	// DCCUR_SETTING

		{	// NORMAL BEHAVIOUR

			// permanent magnets flux vector estimation, position and speed reading
#ifndef APP_INT
			McrpLibf_FluxEst(vam, vbm, iam, ibm);
			FluxPhase=McrpLibf_FluxEst_GetPhase();
			Speed_est=McrpLibf_FluxEst_GetSpeed();
#else	// APP_INT
			McrpLibf_FluxEstA(vam, vbm, iam, ibm, omrif);
			FluxPhase=McrpLibf_FluxEstA_GetPhase();
			Speed_est=McrpLibf_FluxEstA_GetSpeed();
#endif	// APP_INT

			// phase compensation and optional phase offset
			Phase_est=McrpLibf_AngleNrm(FluxPhase+(COMPTIME*Speed_est)+Phase_offset);

			// position loss control
#ifdef TRIP_CONTROL
			MC_TripCon();
#endif	// TRIP_CONTROL

			// speed reference filter
			f_omrif+=(omrif-f_omrif)*OMR_UKT;		// speed reference filtering

			// voltage margin for flux weakening calc
			vmarg=vfmax*K_MARGV1-amp_v;
			if((omegae>0.0f) && (vqc<=0.0f))
				vmarg-=vqc;
			else if((omegae<0.0f) && (vqc>=0.0f))
				vmarg+=vqc;
			if((vmarg<0.0f) && (((omegae>0.0f) && (iqr<=0.0f)) || ((omegae<0.0f) && (iqr>=0.0f))))
				vmarg=0.0f;

			// STARTUP
			if(ramp_status==STOP_W) {	// stop control
				iqr=0.0f;
				idr=0.0f;
				startup_cnt=0;
   				SET_STOPPED
				SystemPhase=0.0f;
				omf=omegae=0.0f;
			}
			else if(startup_cnt==0) {	// alignment
				iqr=0.0f;
				SystemPhase=0.0f;
				omf=omegae=0.0f;
				if(idr<i_start)
					idr+=is_rup;
				else {
					idr=i_start;
					startup_cnt=1;
					FluxPhase=SystemPhase;
					Phase_est=SystemPhase;
					Speed_est=omegae;
#ifndef APP_INT
					McrpLibf_FluxEst_SetFlux(FluxPhase, iam, ibm, omegae);
#else	// APP_INT
					McrpLibf_FluxEstA_SetFlux(FluxPhase, vam, vbm, iam, ibm, omegae);
#endif	// APP_INT
					trip_cnt=0;
				}
			}
			else if(startup_cnt==1) {	// speed increasing

				if(ramp_status==START_P) {
					if(omegae<min_speed)
						omegae+=delta_om;
					else
						startup_cnt=2;
				}
				else if(ramp_status==START_N) {
					if(omegae>-min_speed)
						omegae-=delta_om;
					else
						startup_cnt=2;
				}
				omf=omegae;
	    		delta_ph=omegae*((float) SAM_PER_S);
	    		SystemPhase=McrpLibf_AngleNrm(SystemPhase+delta_ph);
			}
#ifdef DEBUG_OPENLOOP
			else if(DEB_OPNLP) {
	    		SystemPhase=McrpLibf_AngleNrm(SystemPhase+delta_ph);
			}
#endif	// DEBUG_OPENLOOP
			else if(startup_cnt==2) {	// reference system change
				startup_cnt=3;

#ifndef APP_INT
				Speed_est=omf=omegae=f_omrif=omrif;
				McrpLibf_FluxEst_SetFlux(FluxPhase, iam, ibm, omegae);
#else	// APP_INT
				omf=omegae=f_omrif=omrif=Speed_est;
#endif	// APP_INT

				// control memories calc and setting
				McrpLibf_dq_alphabeta(idint, iqint, &iaint, &ibint);// current PI integral terms in old ref. frame
				McrpLibf_dq_alphabeta(idr, iqr, &iar, &ibr);		// references in old ref. frame
				McrpLibf_AngleSet(Phase_est);						// phase update
				McrpLibf_alphabeta_dq(iaint, ibint, &idint, &iqint);// current PI integral terms in new ref. frame
				McrpLibf_alphabeta_dq(iar, ibr, &idr, &iqr);		// references in new ref. frame
				errint=iqr;											// speed PI integral term

				// phase alignment
				SystemPhase=Phase_est;
				trip_cnt=0;
				SET_STARTED
			}
			else if(startup_cnt==3) {	// id decreasing

				// phase and speed update
				SystemPhase=Phase_est;
				omegae=Speed_est;

				// requested id reference calc
				idrvar=0.0f;

				// maximum q current calc
				//iqmax=MC_Vcomp(i_max, idrvar);
				iqmax=i_max;

				// speed control
				iqr=MC_SpeedPI(f_omrif, omegae, iqmax, &errint);

				// idr ramp
				if(idr>(idrvar+is_rdw))
					idr-=is_rdw;
				else if(idr<(idrvar-is_rdw))
					idr+=is_rdw;
				else {
					idr=idrvar;
					++startup_cnt;
				}

				// idr clamping
				af=MC_Vcomp(i_max, iqr);
				if(idr>af)
					idr=af;
				else if(idr<-af)
					idr=-af;
			}
			// STARTUP IS FINISHED, NORMAL CONTROL
			else {		// normal sensorless control

				// phase and speed update
				SystemPhase=Phase_est;
				omegae=Speed_est;

				// id reference calc
				idrvar=0.0f;

				// idr and iqmax calc
#ifdef FLUXWEAK
				MC_FluxWeak(vmarg, id_max, idrvar, &idr);
#else	// FLUXWEAK
				idr=idrvar;
#endif	// FLUXWEAK
				iqmax=MC_Vcomp(i_max, idr);

				// speed control
#ifdef VB_RAMP
				if(RAMPLIM)
					errint*=K_RAMPLIM;
#endif	// VB_RAMP
				iqr=MC_SpeedPI(f_omrif, omegae, iqmax, &errint);

			}

		}	// end of startup and normal behaviour

		// visualization filters
		idmf+=(idm-idmf)*VISU_UKT;					// d current
		iqmf+=(iqm-iqmf)*VISU_UKT;					// q current
		vdmf+=(vdm-vdmf)*VISU_UKT;					// d voltage
		vqmf+=(vqm-vqmf)*VISU_UKT;					// q voltage
		omf+=(omegae-omf)*VISU_UKT;					// angular speed

	}	// if(COM_ON)

	else {

		// position and speed reset
		SystemPhase=0.0f;
		omegae=0.0f;

		// speed PI integral term reset
		errint=0.0f;

		// current references reset
		idr=0.0f;
		iqr=0.0f;

		// measurements reset
		amp_i=0.0f;
		//arg_i=SystemPhase;
		amp_v=0.0f;
		//arg_v=SystemPhase;

		// voltage margin for flux weakening reset
		vmarg=vfmax;

		// phase detection variables reset
		FluxPhase=SystemPhase;
		Phase_est=SystemPhase;
		Speed_est=omegae;
#ifndef APP_INT
		McrpLibf_FluxEst_SetFlux(FluxPhase, iam, ibm, omegae);
#else	// APP_INT
		McrpLibf_FluxEstA_SetFlux(FluxPhase, vam, vbm, iam, ibm, omegae);
#endif	// APP_INT
		trip_cnt=0;

		// startup variables and flags reset
		startup_cnt=0;
		idrvar=i_start;
		iqmax=0.0f;

		// filters memories reset
		f_omrif=0.0f;
		idmf=0.0f;
		iqmf=0.0f;
		vdmf=0.0f;
		vqmf=0.0f;
		omf=0.0f;

	}	// else of if(COM_ON)

	// debug PWM output
#ifdef CPI_TUN
	if(CURPI_TUN) {
		aus=(unsigned short) (8192.0f+49152.0f*idm/i_start);
		MC_SetDBPWM2_TP5(aus);
		aus=(unsigned short) (8192.0f+49152.0f*idr/i_start);
		MC_SetDBPWM3_TP6(aus);
	}
	else
#endif	// CPI_TUN
#ifdef CPI_AUT
	if(CURPI_AUT) {
		aus=(unsigned short) (8192.0f+49152.0f*amp_i/i_start);
		MC_SetDBPWM2_TP5(aus);
		MC_SetDBPWM3_TP6(0);
	}
	else
#endif	// CPI_AUT
#ifdef MOT_IDENT
	if(AUTO_IDEN) {
		aus=(unsigned short) (8192.0f+49152.0f*amp_i/i_max);
		MC_SetDBPWM2_TP5(aus);
		aus=(unsigned short) (8192.0f+49152.0f*amp_v/VB_NOM_VOL);
		MC_SetDBPWM3_TP6(aus);
	}
	else
#endif	// MOT_IDENT
	{	// normal mode
		af=SystemPhase;
		if(af<0.0f)
			af+=FTWOPI;
		aus=(unsigned short) (af*((float)(65536.0/TWOPI)));
		MC_SetDBPWM2_TP5(aus);
#ifdef DEBUG_OPENLOOP
		if(DEB_OPNLP)
			aus=(unsigned short) (8192.0f+49152.0f*amp_i/i_max);
		else
#endif	// DEBUG_OPENLOOP
			aus=(unsigned short) (32768.0f+24576.0f*iqm/i_max);
		MC_SetDBPWM3_TP6(aus);
	}

#ifdef OSC_WIN
	// management of oscilloscope window in GUI
	if(NORM_MODE) {
		if(CSELD)
			sample=ium*1000;				// mA
		else
			sample=SystemPhase*1000;
		if(COM_ON) {
			if(NOT_TRGED) {
				bcntm=(((unsigned short)UIF_W.var.sel) >> 8);
				if(0==(((unsigned short)UIF_W.var.sel) & 0x00FF))
					RES_CSELD
				else
					SET_CSELD
				if(	((omegae>0.0f) && (phmem<0.0f) && (SystemPhase>=0.0f)) ||
					((omegae<0.0f) && (phmem>0.0f) && (SystemPhase<=0.0f)) ) {
					SET_TRGED
					bcnt1=1;
					bcnt0=1;
					wbuf[0]=sample;
				}
			}
			else if(bcnt0<OUTBUF_SDIM) {
				if(bcnt1<bcntm)
					++bcnt1;
				else {
					bcnt1=1;
					wbuf[bcnt0++]=sample;
				}
			}
			else if(NOT_RBUSY) {
				pbuf=rbuf;
				rbuf=wbuf;
				wbuf=pbuf;
				RES_TRGED
			}
		}	// COM_ON
		else {
			RES_TRGED
			bcntm=(((unsigned short)UIF_W.var.sel) >> 8);
			if(0==(((unsigned short)UIF_W.var.sel) & 0x00FF))
				RES_CSELD
			else
				SET_CSELD
			if(bcnt0<OUTBUF_SDIM)
				wbuf[bcnt0++]=0;
			else if(NOT_RBUSY) {
				pbuf=rbuf;
				rbuf=wbuf;
				wbuf=pbuf;
				bcnt0=0;
			}
		}	// COM_OFF
		phmem=SystemPhase;
	}	// NORM_MODE
#endif	// OSC_WIN

	// serial communication polling
	poll_232_pc();

	// main loop synchronization counter
	if(cnt_int)
		--cnt_int;

	// timing debug
	CLR_DL3

	// interrupt end
	return;

}

void MC_FaultInt(void)
/***************************************************************************
*	Description:	interrupt routine for overcurrent fault
***************************************************************************/
{
	MTU.TOERB.BYTE=0xC0;		// all outputs disabled (at inactive level)
	UIF_R.var.all=FAULT_ALL;
	SET_ALRM_ON
	MC_StopOut();

	ICU.IER[0x15].BIT.IEN3=0;	// interrupt disable

	POE.ICSR2.BIT.PIE2=0;		// interrupt request disable
	if(POE.ICSR2.BIT.POE4F==1)
		POE.ICSR2.BIT.POE4F=0;	// reset flag

	POE.OCSR2.BIT.OIE2=0;		// interrupt request disable
	if(POE.OCSR2.BIT.OSF2==1)
		POE.OCSR2.BIT.OSF2=0;	// reset flag
}

